from fastapi import APIRouter, Depends, HTTPException
from typing import Any, List, Dict
from mcpstore.core.store import McpStore
from mcpstore.scripts.models import ToolExecutionRequest, ToolExecutionResponse, ContentUnion, ContentType, Content, TextContent, ImageContent, JsonContent, BinaryContent
from mcpstore.scripts.deps import get_store

router = APIRouter()

@router.post("/execute", response_model=ToolExecutionResponse)
async def execute_tool(request: ToolExecutionRequest, store: McpStore = Depends(get_store)):
    try:
        return await store.execute_tool(
            service_name=request.service_name,
            tool_name=request.tool_name,
            parameters=request.parameters,
            agent_id=getattr(request, "agent_id", None)
        )
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))

def process_result_item(item: Any) -> List[ContentUnion]:
    """处理单个结果项，转换为适当的内容类型"""
    if isinstance(item, str):
        return [TextContent(text=item)]
    elif isinstance(item, bytes):
        return [BinaryContent(data=item)]
    elif isinstance(item, dict):
        return [JsonContent(data=item)]
    elif isinstance(item, (list, tuple)):
        results = []
        for sub_item in item:
            results.extend(process_result_item(sub_item))
        return results
    else:
        # 尝试转换为字符串
        return [TextContent(text=str(item))] 



#!/usr/bin/env python
"""
MCP独立运行入口点
可以直接运行: python -m src.mcpstore.cli.mcp_standalone
"""

import asyncio
import sys
from src.mcpstore.cli.standalone import main

if __name__ == "__main__":
    # 在Windows平台上设置事件循环策略
    if sys.platform == 'win32':
        asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
    
    # 运行主函数
    asyncio.run(main()) 



#!/usr/bin/env python
"""
独立运行MCP服务注册与连接
不依赖FastAPI，可直接从命令行启动
"""

import asyncio
import logging
import os
import json
import sys
import argparse
from pathlib import Path
from typing import Dict, Any, List, Optional

# 导入必要的模块
try:
    # 尝试直接导入(相对导入)
    from mcpstore.core.orchestrator import MCPOrchestrator
    from mcpstore.core.registry import ServiceRegistry
    from mcpstore.plugins.json_mcp import MCPConfig
    from mcpstore.core.models.service import TransportType
except ImportError:
    # 尝试从src导入(绝对导入)
    from src.mcpstore.core.orchestrator import MCPOrchestrator
    from src.mcpstore.core.registry import ServiceRegistry
    from src.mcpstore.plugins.json_mcp import MCPConfig
    from src.mcpstore.core.models.service import TransportType

from fastmcp import Client
from fastmcp.client.transports import (
    StdioTransport,
    NodeStdioTransport,
    PythonStdioTransport
)

# 日志配置
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)

logger = logging.getLogger(__name__)

class MCPStandalone:
    """MCP独立运行模式"""
    
    def __init__(self, config_path: str = None, debug: bool = False):
        """
        初始化MCP独立运行模式
        
        Args:
            config_path: MCP配置文件路径，默认使用标准路径
            debug: 是否启用调试模式
        """
        # 设置日志级别
        if debug:
            logging.getLogger().setLevel(logging.DEBUG)
            # 启用所有相关日志
            loggers = [
                'fastmcp', 'mcp', 'httpx', 'asyncio',
                'mcp.client.stdio', 'mcp.client.transports',
                'mcpstore'
            ]
            for logger_name in loggers:
                logging.getLogger(logger_name).setLevel(logging.DEBUG)
        
        # 创建配置对象
        self.config = {
            "timing": {
                "heartbeat_interval_seconds": 60,
                "heartbeat_timeout_seconds": 180,
                "reconnection_interval_seconds": 60,
                "http_timeout_seconds": 10,
                "command_timeout_seconds": 10
            }
        }
        
        # 创建MCP配置
        self.mcp_config = MCPConfig(json_path=config_path)
        
        # 创建服务注册表
        self.registry = ServiceRegistry()
        
        # 创建编排器
        self.orchestrator = MCPOrchestrator(self.config, self.registry)
        self.orchestrator.mcp_config = self.mcp_config
    
    async def register_services(self) -> Dict[str, Any]:
        """
        注册MCP配置中的所有服务
        
        Returns:
            注册结果字典
        """
        config = self.mcp_config.load_config()
        logger.info(f"加载了MCP配置: {len(config.get('mcpServers', {}))}个服务")
        
        results = await self.orchestrator.register_json_services(config)
        
        # 输出结果摘要
        total_success = results.get("total_success", 0)
        total_failure = results.get("total_failure", 0)
        logger.info(f"服务注册完成: 成功 {total_success}, 失败 {total_failure}")
        
        # 如果有成功的服务，打印服务和工具信息
        if total_success > 0:
            print("\n=== 已注册的服务和工具 ===")
            for service_name, service_result in results.get("services", {}).items():
                if service_result.get("status") == "success":
                    print(f"服务: {service_name}")
                    
                    # 获取该服务的工具列表
                    tools = self.registry.get_tools_for_service(service_name)
                    if tools:
                        print(f"  工具 ({len(tools)}):")
                        for tool_name in tools:
                            print(f"    - {tool_name}")
                    print()
        
        return results
    
    async def connect_service(self, service_name: str) -> bool:
        """
        连接到指定的MCP服务
        
        Args:
            service_name: 服务名称
            
        Returns:
            连接是否成功
        """
        # 获取服务配置
        service_config = self.mcp_config.get_service_config(service_name)
        
        if not service_config:
            logger.error(f"服务 '{service_name}' 不存在于配置中")
            return False
        
        logger.info(f"尝试连接服务: {service_name}")
        
        # 根据服务类型选择连接方法
        if "url" in service_config:
            # URL服务，使用HTTP连接
            success, client = await self.orchestrator.is_service_healthy(service_name, service_config)
            if success:
                logger.info(f"成功连接到URL服务: {service_name}")
                async with client:
                    tools = await client.list_tools()
                    logger.info(f"获取到 {len(tools)} 个工具")
                    for tool in tools:
                        logger.info(f"  工具: {tool.name}")
                return True
            else:
                logger.error(f"连接URL服务失败: {service_name}")
                return False
        elif "command" in service_config:
            # Studio服务，使用connect_studio_service方法
            success, client, tool_definitions = await self.orchestrator.connect_studio_service(
                service_name, service_config
            )
            if success:
                logger.info(f"成功连接到Studio服务: {service_name}")
                logger.info(f"获取到 {len(tool_definitions)} 个工具")
                for tool in tool_definitions:
                    logger.info(f"  工具: {tool.get('name')}")
                return True
            else:
                logger.error(f"连接Studio服务失败: {service_name}")
                return False
        else:
            logger.error(f"未知的服务类型: {service_name}")
            return False

    async def test_direct_node_transport(self, script_path: str, env: Optional[Dict[str, str]] = None) -> bool:
        """
        直接测试NodeStdioTransport
        
        Args:
            script_path: Node.js脚本路径
            env: 环境变量
            
        Returns:
            测试是否成功
        """
        try:
            logger.info(f"直接测试NodeStdioTransport: {script_path}")
            
            # 默认环境变量
            if env is None:
                env = {'DEBUG': '*'}
            
            # 使用NodeStdioTransport
            transport = NodeStdioTransport(
                command_args=[script_path],
                env=env
            )
            
            logger.info("创建Client...")
            client = Client(transport)
            
            logger.info("尝试连接...")
            async with client:
                logger.info("连接成功！")
                
                # 尝试列出工具
                logger.info("列出工具...")
                tools = await client.list_tools()
                logger.info(f"成功连接，找到 {len(tools)} 个工具")
                
                for tool in tools:
                    logger.info(f"  工具: {tool.name}: {tool.description}")
                
                return True
        except Exception as e:
            logger.error(f"连接失败: {e}")
            import traceback
            traceback.print_exc()
            return False


# 为Typer命令行创建的异步执行函数
async def run_standalone(config_path: str = None, 
                        debug: bool = False,
                        service_name: str = None,
                        test_node_path: str = None) -> int:
    """
    运行独立模式的异步函数
    
    Args:
        config_path: 配置文件路径
        debug: 是否启用调试模式
        service_name: 要测试的服务名称
        test_node_path: 要测试的Node.js脚本路径
        
    Returns:
        退出码：0表示成功，1表示失败
    """
    # 设置Windows事件循环策略
    if sys.platform == 'win32':
        asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
        logger.info("已设置Windows ProactorEventLoop策略")
    
    # 创建MCP独立运行实例
    mcp = MCPStandalone(config_path=config_path, debug=debug)
    
    # 执行相应的操作
    if test_node_path:
        # 直接测试Node.js脚本
        success = await mcp.test_direct_node_transport(test_node_path)
        return 0 if success else 1
    elif service_name:
        # 连接特定服务
        success = await mcp.connect_service(service_name)
        return 0 if success else 1
    else:
        # 注册所有服务
        results = await mcp.register_services()
        return 0 if results.get("total_success", 0) > 0 else 1


# 为兼容直接调用而保留的main函数
async def main():
    """传统命令行参数的主函数"""
    # 解析命令行参数
    parser = argparse.ArgumentParser(description='MCP独立运行模式')
    parser.add_argument('--config', '-c', help='MCP配置文件路径')
    parser.add_argument('--debug', '-d', action='store_true', help='启用调试模式')
    parser.add_argument('--service', '-s', help='要连接的特定服务名称')
    parser.add_argument('--test-node', '-t', help='直接测试指定的Node.js脚本路径')
    
    args = parser.parse_args()
    
    # 调用共享的异步执行函数
    exit_code = await run_standalone(
        config_path=args.config,
        debug=args.debug,
        service_name=args.service,
        test_node_path=args.test_node
    )
    
    sys.exit(exit_code)


if __name__ == "__main__":
    # 运行主函数
    asyncio.run(main()) 



import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from dataclasses import dataclass
from typing import Dict, Any, Optional, AsyncGenerator, List
import uuid
import httpx
import json
import logging
import asyncio
from urllib.parse import urljoin

logger = logging.getLogger(__name__)

@dataclass
class StreamableHTTPConfig:
    """Streamable HTTP传输配置"""
    base_url: str
    timeout: int = 30
    session_id: Optional[str] = None
    retry_attempts: int = 3
    retry_delay: float = 1.0
    event_id_header: str = "Last-Event-ID"
    session_id_header: str = "Mcp-Session-Id"

class StreamableHTTPTransport:
    """实现MCP Streamable HTTP传输协议
    
    基于MCP 2025-03-26版本规范，提供统一的双向通信能力。
    支持会话管理、连接恢复和向后兼容。
    """
    
    # 方法名映射，将简化名称映射到服务器期望的格式
    METHOD_MAPPING = {
        "list_tools": "tools/list",
        "call_tool": "tools/call",
        "initialize": "initialize",
        "ping": "ping"
        # 可以根据需要添加更多映射
    }
    
    def __init__(self, config: StreamableHTTPConfig):
        self.config = config
        self.client = httpx.AsyncClient(timeout=config.timeout, follow_redirects=True)
        self.last_event_id: Optional[str] = None
        
    async def initialize(self) -> Dict[str, Any]:
        """初始化连接并获取会话ID
        
        发送初始化请求，建立会话，并返回服务器响应。
        
        Returns:
            Dict[str, Any]: 服务器的初始化响应
        """
        headers = {
            "Accept": "application/json, text/event-stream",
            "Content-Type": "application/json"
        }
        
        request_id = str(uuid.uuid4())
        # 确保使用正确的方法名（initialize 不需要映射，但为了一致性，我们仍然从映射中获取）
        method = "initialize"
        server_method = self.METHOD_MAPPING.get(method, method)
        
        payload = {
            "jsonrpc": "2.0", 
            "method": server_method, 
            "params": {
                "clientInfo": {
                    "name": "mcp-client",
                    "version": "1.0.0"
                },
                "protocolVersion": "2025-03-26",  # 添加协议版本
                "capabilities": {                 # 添加客户端能力
                    "streaming": True,
                    "json": True,
                    "binary": False
                }
            },
            "id": request_id
        }
        
        try:
            logger.debug(f"Initializing connection with method={server_method}")
            response = await self.client.post(
                urljoin(self.config.base_url, "/mcp"),
                headers=headers,
                json=payload
            )
            response.raise_for_status()
            
            # 获取并保存会话ID
            session_id = response.headers.get(self.config.session_id_header)
            if session_id:
                self.config.session_id = session_id
                logger.info(f"Session established with ID: {session_id}")
            
            # 处理响应内容
            if response.content:
                try:
                    return response.json()
                except json.JSONDecodeError:
                    logger.warning(f"Failed to parse response as JSON: {response.content}")
                    # 返回一个默认的成功响应，避免中断流程
                    return {"status": "connected", "session_id": session_id or "unknown"}
            else:
                logger.warning("Empty response received from server")
                # 返回一个默认的成功响应，避免中断流程
                return {"status": "connected", "session_id": session_id or "unknown"}
            
        except httpx.HTTPStatusError as e:
            logger.error(f"HTTP error during initialization: {e.response.status_code} {e.response.reason_phrase}")
            raise
        except Exception as e:
            logger.error(f"Error during transport initialization: {e}")
            raise
    
    async def call_tool(self, tool_name: str, tool_args: Dict[str, Any]) -> Any:
        """调用工具方法
        
        使用Streamable HTTP协议调用指定的工具，并返回结果。
        此方法与registry.py中定义的SessionProtocol接口兼容。
        
        Args:
            tool_name: 工具名称
            tool_args: 工具参数
            
        Returns:
            Any: 工具执行结果
        """
        logger.info(f"Calling tool '{tool_name}' with args: {tool_args}")
        
        try:
            # 发送工具调用请求
            responses = []
            # 使用 call_tool 作为方法名，会被映射到 tools/call
            method = "call_tool"
            params = {"name": tool_name, "arguments": tool_args}
            
            async for response in self.send_request(method, params):
                responses.append(response)
                # 只获取第一个响应
                break
                
            if not responses:
                logger.warning(f"No response received from tool '{tool_name}'")
                return {"content": [{"text": f"No response received from tool '{tool_name}'"}]}
                
            result = responses[0]
            
            # 格式化响应为兼容格式
            if isinstance(result, dict) and "result" in result:
                # 如果响应中有result字段，将其作为文本内容返回
                return {"content": [{"text": str(result["result"])}]}
            elif isinstance(result, dict) and "error" in result:
                # 如果响应中有error字段，将其作为错误信息返回
                error_msg = result.get("error", {}).get("message", "Unknown error")
                return {"content": [{"text": f"Error: {error_msg}"}]}
            else:
                # 其他情况，直接返回响应
                return {"content": [{"text": str(result)}]}
                
        except Exception as e:
            logger.error(f"Error calling tool '{tool_name}': {e}", exc_info=True)
            return {"content": [{"text": f"Error calling tool '{tool_name}': {str(e)}"}]}
        
    async def send_request(self, method: str, params: Dict[str, Any]) -> AsyncGenerator[Dict[str, Any], None]:
        """发送请求并处理流式响应
        
        Args:
            method: 请求方法名
            params: 请求参数
            
        Yields:
            Dict[str, Any]: 服务器响应数据流
        """
        headers = {
            "Accept": "application/json, text/event-stream",
            "Content-Type": "application/json"
        }
        
        if self.config.session_id:
            headers[self.config.session_id_header] = self.config.session_id
            
        if self.last_event_id:
            headers[self.config.event_id_header] = self.last_event_id
        
        # 将简化的方法名转换为服务器期望的格式
        server_method = self.METHOD_MAPPING.get(method, method)
        if server_method != method:
            logger.debug(f"Mapping method name from '{method}' to '{server_method}'")
            
        request_id = str(uuid.uuid4())
        payload = {
            "jsonrpc": "2.0",
            "method": server_method,
            "params": params,
            "id": request_id
        }
        
        try:
            logger.debug(f"Sending request: method={server_method}, params={params}")
            async with self.client.stream(
                "POST",
                urljoin(self.config.base_url, "/mcp"),
                headers=headers,
                json=payload
            ) as response:
                response.raise_for_status()
                
                content_type = response.headers.get("Content-Type", "")
                
                if "text/event-stream" in content_type:
                    # 处理SSE流
                    buffer = ""
                    async for chunk in response.aiter_text():
                        buffer += chunk
                        
                        while "\n\n" in buffer:
                            message, buffer = buffer.split("\n\n", 1)
                            event_data = {}
                            
                            for line in message.split("\n"):
                                if not line or line.startswith(":"):
                                    continue  # 忽略注释和空行
                                    
                                if ":" in line:
                                    field, value = line.split(":", 1)
                                    value = value.lstrip()  # 移除前导空格
                                    
                                    if field == "id":
                                        self.last_event_id = value
                                    elif field == "data":
                                        try:
                                            event_data = json.loads(value)
                                        except json.JSONDecodeError:
                                            logger.warning(f"Failed to parse SSE data: {value}")
                                
                            if event_data:
                                yield event_data
                else:
                    # 处理普通JSON响应 - 修复方法，读取完整响应内容
                    try:
                        # 读取完整响应内容而不是直接调用response.json()
                        content = await response.aread()
                        data = json.loads(content)
                        yield data
                    except json.JSONDecodeError:
                        logger.error(f"Failed to parse response as JSON: {content}")
                        raise
                    
        except httpx.HTTPStatusError as e:
            logger.error(f"HTTP error during request: {e.response.status_code} {e.response.reason_phrase}")
            raise
        except Exception as e:
            logger.error(f"Error during request processing: {e}")
            raise
    
    async def send_notification(self, method: str, params: Dict[str, Any]) -> None:
        """发送通知（不需要响应的请求）
        
        Args:
            method: 通知方法名
            params: 通知参数
        """
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        
        if self.config.session_id:
            headers[self.config.session_id_header] = self.config.session_id
            
        payload = {
            "jsonrpc": "2.0",
            "method": method,
            "params": params
        }
        
        try:
            response = await self.client.post(
                urljoin(self.config.base_url, "/mcp"),
                headers=headers,
                json=payload
            )
            
            if response.status_code != 202:
                logger.warning(f"Unexpected status code for notification: {response.status_code}")
                
        except Exception as e:
            logger.error(f"Error sending notification: {e}")
            raise
    
    async def listen_server(self) -> AsyncGenerator[Dict[str, Any], None]:
        """监听服务器发送的消息
        
        打开GET连接以接收服务器主动发送的消息。
        
        Yields:
            Dict[str, Any]: 服务器发送的消息
        """
        headers = {
            "Accept": "text/event-stream"
        }
        
        if self.config.session_id:
            headers[self.config.session_id_header] = self.config.session_id
            
        if self.last_event_id:
            headers[self.config.event_id_header] = self.last_event_id
        
        try:
            async with self.client.stream(
                "GET",
                urljoin(self.config.base_url, "/mcp"),
                headers=headers
            ) as response:
                response.raise_for_status()
                
                if response.status_code == 405:
                    logger.warning("Server does not support GET requests for listening")
                    return
                
                buffer = ""
                async for chunk in response.aiter_text():
                    buffer += chunk
                    
                    while "\n\n" in buffer:
                        message, buffer = buffer.split("\n\n", 1)
                        event_data = {}
                        
                        for line in message.split("\n"):
                            if not line or line.startswith(":"):
                                continue  # 忽略注释和空行
                                
                            if ":" in line:
                                field, value = line.split(":", 1)
                                value = value.lstrip()  # 移除前导空格
                                
                                if field == "id":
                                    self.last_event_id = value
                                elif field == "data":
                                    try:
                                        event_data = json.loads(value)
                                    except json.JSONDecodeError:
                                        logger.warning(f"Failed to parse SSE data: {value}")
                            
                        if event_data:
                            yield event_data
                            
        except httpx.HTTPStatusError as e:
            if e.response.status_code == 405:
                logger.warning("Server does not support GET requests for listening")
            else:
                logger.error(f"HTTP error during listening: {e.response.status_code} {e.response.reason_phrase}")
            raise
        except Exception as e:
            logger.error(f"Error during server listening: {e}")
            raise
                
    async def close(self) -> None:
        """关闭连接并清理资源
        
        如果有会话ID，尝试显式终止会话。
        """
        if self.config.session_id:
            try:
                headers = {self.config.session_id_header: self.config.session_id}
                await self.client.delete(
                    urljoin(self.config.base_url, "/mcp"),
                    headers=headers
                )
                logger.info(f"Session {self.config.session_id} terminated")
            except Exception as e:
                logger.warning(f"Failed to terminate session: {e}")
                
        await self.client.aclose()
        logger.info("Transport resources cleaned up") 
 


"""
MCPStore CLI模块
提供命令行界面功能
"""

try:
    # 尝试直接导入
    from mcpstore.cli.standalone import MCPStandalone
except ImportError:
    # 尝试从src导入
    from src.mcpstore.cli.standalone import MCPStandalone

__all__ = ["MCPStandalone"] 



import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import logging
from datetime import datetime
from typing import Dict, Any, Optional, Tuple, List, Set, TypeVar, Generic, Protocol

logger = logging.getLogger(__name__)

# 定义一个协议，表示任何具有call_tool方法的会话类型
class SessionProtocol(Protocol):
    async def call_tool(self, tool_name: str, tool_args: Dict[str, Any]) -> Any:
        ...

# 会话类型变量
SessionType = TypeVar('SessionType')

class ServiceRegistry:
    """
    Manages the state of connected services and their tools, with agent_id isolation.
    
    agent_id 作为一级 key，实现 store/agent/agent 之间的完全隔离：
    - self.sessions: Dict[agent_id, Dict[service_name, session]]
    - self.tool_cache: Dict[agent_id, Dict[tool_name, tool_def]]
    - self.tool_to_session_map: Dict[agent_id, Dict[tool_name, session]]
    - self.service_health: Dict[agent_id, Dict[service_name, last_heartbeat]]
    所有操作都必须带 agent_id，store 级别用 main_client，agent 级别用实际 agent_id。
    """
    def __init__(self):
        # agent_id -> {service_name: session}
        self.sessions: Dict[str, Dict[str, Any]] = {}
        # agent_id -> {service_name: last_heartbeat_time}
        self.service_health: Dict[str, Dict[str, datetime]] = {}
        # agent_id -> {tool_name: tool_definition}
        self.tool_cache: Dict[str, Dict[str, Dict[str, Any]]] = {}
        # agent_id -> {tool_name: session}
        self.tool_to_session_map: Dict[str, Dict[str, Any]] = {}
        logger.info("ServiceRegistry initialized (multi-context isolation).")

    def clear(self, agent_id: str):
        """
        清空指定 agent_id 的所有注册服务和工具。
        只影响该 agent_id 下的服务、工具、会话，不影响其它 agent。
        """
        self.sessions.pop(agent_id, None)
        self.service_health.pop(agent_id, None)
        self.tool_cache.pop(agent_id, None)
        self.tool_to_session_map.pop(agent_id, None)

    def add_service(self, agent_id: str, name: str, session: Any, tools: List[Tuple[str, Dict[str, Any]]]) -> List[str]:
        """
        为指定 agent_id 注册服务及其工具。
        - agent_id: store/agent 的唯一标识
        - name: 服务名
        - session: 服务会话对象
        - tools: [(tool_name, tool_def)]
        返回实际注册的工具名列表。
        """
        if agent_id not in self.sessions:
            self.sessions[agent_id] = {}
        if agent_id not in self.service_health:
            self.service_health[agent_id] = {}
        if agent_id not in self.tool_cache:
            self.tool_cache[agent_id] = {}
        if agent_id not in self.tool_to_session_map:
            self.tool_to_session_map[agent_id] = {}
            
        # 只在首次注册时打印日志
        if name not in self.sessions[agent_id]:
            print(f"[DEBUG][add_service] 首次注册服务 - agent_id={agent_id}, name={name}")
        
        if name in self.sessions[agent_id]:
            logger.warning(f"Attempting to add already registered service: {name} for agent {agent_id}. Removing old service before overwriting.")
            self.remove_service(agent_id, name)
            
        self.sessions[agent_id][name] = session
        self.service_health[agent_id][name] = datetime.now() # Mark healthy on add
        added_tool_names = []
        for tool_name, tool_definition in tools:
            if not tool_name.startswith(f"{name}_"):
                logger.warning(f"Tool '{tool_name}' does not belong to service '{name}'. Skipping this tool.")
                continue
            if tool_name in self.tool_cache[agent_id]:
                existing_session = self.tool_to_session_map[agent_id].get(tool_name)
                if existing_session is not session:
                    logger.warning(f"Tool name conflict: '{tool_name}' from {name} for agent {agent_id} conflicts with existing tool. Skipping this tool.")
                    continue
            self.tool_cache[agent_id][tool_name] = tool_definition
            self.tool_to_session_map[agent_id][tool_name] = session
            added_tool_names.append(tool_name)
        logger.info(f"Service '{name}' for agent '{agent_id}' added with tools: {added_tool_names}")
        return added_tool_names

    def remove_service(self, agent_id: str, name: str) -> Optional[Any]:
        """
        移除指定 agent_id 下的服务及其所有工具。
        只影响该 agent_id，不影响其它 agent。
        """
        session = self.sessions.get(agent_id, {}).pop(name, None)
        if not session:
            logger.warning(f"Attempted to remove non-existent service: {name} for agent {agent_id}")
            return None
        if agent_id in self.service_health and name in self.service_health[agent_id]:
            del self.service_health[agent_id][name]
        # Remove associated tools efficiently
        tools_to_remove = [tool_name for tool_name, owner_session in self.tool_to_session_map.get(agent_id, {}).items() if owner_session is session]
        for tool_name in tools_to_remove:
            if tool_name in self.tool_cache.get(agent_id, {}): del self.tool_cache[agent_id][tool_name]
            if tool_name in self.tool_to_session_map.get(agent_id, {}): del self.tool_to_session_map[agent_id][tool_name]
        logger.info(f"Service '{name}' for agent '{agent_id}' removed from registry.")
        return session

    def get_session(self, agent_id: str, name: str) -> Optional[Any]:
        """
        获取指定 agent_id 下的服务会话。
        """
        return self.sessions.get(agent_id, {}).get(name)

    def get_session_for_tool(self, agent_id: str, tool_name: str) -> Optional[Any]:
        """
        获取指定 agent_id 下工具对应的服务会话。
        """
        return self.tool_to_session_map.get(agent_id, {}).get(tool_name)

    def get_all_tools(self, agent_id: str) -> List[Dict[str, Any]]:
        """
        获取指定 agent_id 下所有工具的定义。
        """
        all_tools = []
        for tool_name, tool_def in self.tool_cache.get(agent_id, {}).items():
            session = self.tool_to_session_map.get(agent_id, {}).get(tool_name)
            service_name = None
            for name, sess in self.sessions.get(agent_id, {}).items():
                if sess is session:
                    service_name = name
                    break
            tool_with_service = tool_def.copy()
            if "function" not in tool_with_service and isinstance(tool_with_service, dict):
                tool_with_service = {
                    "type": "function",
                    "function": tool_with_service
                }
            if "function" in tool_with_service:
                function_data = tool_with_service["function"]
                if service_name:
                    original_description = function_data.get("description", "")
                    if not original_description.endswith(f" (来自服务: {service_name})"):
                        function_data["description"] = f"{original_description} (来自服务: {service_name})"
                function_data["service_info"] = {"service_name": service_name}
            all_tools.append(tool_with_service)
        logger.info(f"Returning {len(all_tools)} tools from {len(self.get_all_service_names(agent_id))} services for agent {agent_id}")
        return all_tools

    def get_all_tool_info(self, agent_id: str) -> List[Dict[str, Any]]:
        """
        获取指定 agent_id 下所有工具的详细信息。
        """
        tools_info = []
        for tool_name in self.tool_cache.get(agent_id, {}).keys():
            session = self.tool_to_session_map.get(agent_id, {}).get(tool_name)
            service_name = None
            for name, sess in self.sessions.get(agent_id, {}).items():
                if sess is session:
                    service_name = name
                    break
            detailed_tool = self._get_detailed_tool_info(agent_id, tool_name)
            if detailed_tool:
                detailed_tool["service_name"] = service_name
                tools_info.append(detailed_tool)
        return tools_info

    def get_connected_services(self, agent_id: str) -> List[Dict[str, Any]]:
        """
        获取指定 agent_id 下所有已连接服务的信息。
        """
        services = []
        for name in self.get_all_service_names(agent_id):
            tools = self.get_tools_for_service(agent_id, name)
            services.append({
                "name": name,
                "tool_count": len(tools)
            })
        return services

    def get_tools_for_service(self, agent_id: str, name: str) -> List[str]:
        """
        获取指定 agent_id 下某服务的所有工具名。
        """
        session = self.sessions.get(agent_id, {}).get(name)
        logger.info(f"Getting tools for service: {name} (agent_id={agent_id})")
        
        # 只在调试特定问题时打印详细日志
        if logger.getEffectiveLevel() <= logging.DEBUG:
            print(f"[DEBUG][get_tools_for_service] agent_id={agent_id}, name={name}, id(session)={id(session) if session else None}")
        
        if not session:
            return []
            
        tools = [tool_name for tool_name in self.tool_cache.get(agent_id, {}).keys() if tool_name.startswith(f"{name}_")]
        return tools

    def _extract_description_from_schema(self, prop_info):
        """从 schema 中提取描述信息"""
        if isinstance(prop_info, dict):
            # 优先查找 description 字段
            if 'description' in prop_info:
                return prop_info['description']
            # 其次查找 title 字段
            elif 'title' in prop_info:
                return prop_info['title']
            # 检查是否有 anyOf 或 allOf 结构
            elif 'anyOf' in prop_info:
                for item in prop_info['anyOf']:
                    if isinstance(item, dict) and 'description' in item:
                        return item['description']
            elif 'allOf' in prop_info:
                for item in prop_info['allOf']:
                    if isinstance(item, dict) and 'description' in item:
                        return item['description']

        return "无描述"

    def _extract_type_from_schema(self, prop_info):
        """从 schema 中提取类型信息"""
        if isinstance(prop_info, dict):
            if 'type' in prop_info:
                return prop_info['type']
            elif 'anyOf' in prop_info:
                # 处理 Union 类型
                types = []
                for item in prop_info['anyOf']:
                    if isinstance(item, dict) and 'type' in item:
                        types.append(item['type'])
                return '|'.join(types) if types else '未知'
            elif 'allOf' in prop_info:
                # 处理 intersection 类型
                for item in prop_info['allOf']:
                    if isinstance(item, dict) and 'type' in item:
                        return item['type']

        return "未知"

    def _get_detailed_tool_info(self, agent_id: str, tool_name: str) -> Dict[str, Any]:
        """
        获取指定 agent_id 下某工具的详细信息。
        """
        tool_def = self.tool_cache.get(agent_id, {}).get(tool_name)
        if not tool_def:
            return {}
        session = self.tool_to_session_map.get(agent_id, {}).get(tool_name)
        service_name = None
        if session:
            for name, sess in self.sessions.get(agent_id, {}).items():
                if sess is session:
                    service_name = name
                    break
        if "function" in tool_def:
            function_data = tool_def["function"]
            tool_info = {
                "name": tool_name,
                "description": function_data.get("description", ""),
                "service_name": service_name,
                "inputSchema": function_data.get("parameters", {})
            }
        else:
            tool_info = {
                "name": tool_name,
                "description": tool_def.get("description", ""),
                "service_name": service_name,
                "inputSchema": tool_def.get("parameters", {})
            }
        return tool_info

    def get_service_details(self, agent_id: str, name: str) -> Dict[str, Any]:
        """
        获取指定 agent_id 下某服务的详细信息。
        """
        if name not in self.sessions.get(agent_id, {}):
            return {}
            
        logger.info(f"Getting service details for: {name} (agent_id={agent_id})")
        session = self.sessions.get(agent_id, {}).get(name)
        
        # 只在调试特定问题时打印详细日志
        if logger.getEffectiveLevel() <= logging.DEBUG:
            print(f"[DEBUG][get_service_details] agent_id={agent_id}, name={name}, id(session)={id(session) if session else None}")
            
        tools = self.get_tools_for_service(agent_id, name)
        last_heartbeat = self.service_health.get(agent_id, {}).get(name)
        detailed_tools = []
        for tool_name in tools:
            detailed_tool = self._get_detailed_tool_info(agent_id, tool_name)
            if detailed_tool:
                detailed_tools.append(detailed_tool)
        return {
            "name": name,
            "tools": detailed_tools,
            "tool_count": len(tools),
            "last_heartbeat": str(last_heartbeat) if last_heartbeat else "N/A",
            "connected": name in self.sessions.get(agent_id, {})
        }

    def get_all_service_names(self, agent_id: str) -> List[str]:
        """
        获取指定 agent_id 下所有已注册服务名。
        """
        return list(self.sessions.get(agent_id, {}).keys())

    def update_service_health(self, agent_id: str, name: str):
        """
        更新指定 agent_id 下某服务的心跳时间。
        """
        if name in self.sessions.get(agent_id, {}):
            if agent_id not in self.service_health:
                self.service_health[agent_id] = {}
            self.service_health[agent_id][name] = datetime.now()
            logger.debug(f"Health updated for service: {name} (agent_id={agent_id})")

    def get_last_heartbeat(self, agent_id: str, name: str) -> Optional[datetime]:
        """
        获取指定 agent_id 下某服务的最后心跳时间。
        """
        return self.service_health.get(agent_id, {}).get(name)

    def has_service(self, agent_id: str, name: str) -> bool:
        """
        判断指定 agent_id 下是否存在某服务。
        """
        return name in self.sessions.get(agent_id, {})

    def get_service_config(self, agent_id: str, name: str) -> Optional[Dict[str, Any]]:
        """获取服务配置"""
        if not self.has_service(agent_id, name):
            return None
            
        # 从 orchestrator 的 mcp_config 获取配置
        from api.deps import app_state
        orchestrator = app_state.get("orchestrator")
        if orchestrator and orchestrator.mcp_config:
            return orchestrator.mcp_config.get_service_config(name)
            
        return None



from pydantic import BaseModel
from typing import Optional, List, Dict, Any

class ClientRegistrationRequest(BaseModel):
    client_id: Optional[str] = None
    service_names: Optional[List[str]] = None

class ClientRegistrationResponse(BaseModel):
    client_id: str
    service_names: List[str]
    config: Dict[str, Any] 



import os
import sys

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import logging
from typing import Dict, Any

logger = logging.getLogger(__name__)

# --- Configuration Constants (default values) ---
HEARTBEAT_INTERVAL_SECONDS = 60
HEARTBEAT_TIMEOUT_SECONDS = 180
HTTP_TIMEOUT_SECONDS = 10
RECONNECTION_INTERVAL_SECONDS = 60
REACT_MAX_ITERATIONS = 5
REACT_ENABLE_TRACE = False
STREAMABLE_HTTP_ENDPOINT = "/mcp"

# @dataclass
# class LLMConfig:
#     provider: str = "openai_compatible"
#     api_key: str = ""
#     model: str = ""
#     base_url: Optional[str] = None

# def load_llm_config() -> LLMConfig:
#     """从环境变量加载LLM配置（仅支持openai兼容接口）"""
#     api_key = os.environ.get("OPENAI_API_KEY", "")
#     model = os.environ.get("OPENAI_MODEL", "")
#     base_url = os.environ.get("OPENAI_BASE_URL")
#     provider = "openai_compatible"
#     if not api_key:
#         logger.warning("OPENAI_API_KEY not set in environment.")
#     if not model:
#         logger.warning("OPENAI_MODEL not set in environment.")
#     return LLMConfig(provider=provider, api_key=api_key, model=model, base_url=base_url)

def _get_env_int(var: str, default: int) -> int:
    try:
        return int(os.environ.get(var, default))
    except Exception:
        logger.warning(f"环境变量{var}格式错误，使用默认值{default}")
        return default

def _get_env_bool(var: str, default: bool) -> bool:
    val = os.environ.get(var)
    if val is None:
        return default
    return val.lower() in ("1", "true", "yes", "on")

def load_app_config() -> Dict[str, Any]:
    """从环境变量加载全局配置"""
    config_data = {
        "heartbeat_interval": _get_env_int("HEARTBEAT_INTERVAL_SECONDS", HEARTBEAT_INTERVAL_SECONDS),
        "heartbeat_timeout": _get_env_int("HEARTBEAT_TIMEOUT_SECONDS", HEARTBEAT_TIMEOUT_SECONDS),
        "http_timeout": _get_env_int("HTTP_TIMEOUT_SECONDS", HTTP_TIMEOUT_SECONDS),
        "reconnection_interval": _get_env_int("RECONNECTION_INTERVAL_SECONDS", RECONNECTION_INTERVAL_SECONDS),
        "react_max_iterations": _get_env_int("REACT_MAX_ITERATIONS", REACT_MAX_ITERATIONS),
        "react_enable_trace": _get_env_bool("REACT_ENABLE_TRACE", REACT_ENABLE_TRACE),
        "streamable_http_endpoint": os.environ.get("STREAMABLE_HTTP_ENDPOINT", STREAMABLE_HTTP_ENDPOINT),
    }
    # 加载LLM配置
    # config_data["llm_config"] = load_llm_config()
    # logger.info(f"Loaded configuration from environment: {config_data}")
    return config_data



from fastapi import APIRouter, Depends, HTTPException
from typing import Dict, Any, Optional, List
from mcpstore.core.store import McpStore
from mcpstore.core.models.tool import (
    ToolExecutionRequest, ToolExecutionResponse,
    ToolInfo, ToolsResponse
)
from mcpstore.scripts.deps import get_store
import logging

router = APIRouter()
logger = logging.getLogger(__name__)

@router.get("/tools", response_model=ToolsResponse)
async def list_tools(
    agent_id: Optional[str] = None,
    client_id: Optional[str] = None,
    store: McpStore = Depends(get_store)
):
    """获取工具列表"""
    return store.list_tools()

@router.post("/tools/execute", response_model=ToolExecutionResponse)
async def execute_tool(
    request: ToolExecutionRequest,
    store: McpStore = Depends(get_store)
):
    """执行工具"""
    return await store.execute_tool(request) 



"""
MCPStore Context Module
提供 MCPStore 的上下文管理功能
"""

from typing import Dict, List, Optional, Any, Union
from dataclasses import dataclass
from enum import Enum
from mcpstore.core.models.tool import ToolExecutionRequest, ToolExecutionResponse
from mcpstore.core.models.service import (
    ServiceInfo, AddServiceRequest, ServiceConfigUnion,
    URLServiceConfig, CommandServiceConfig, MCPServerConfig
)

@dataclass
class ServiceInfo:
    """服务信息"""
    name: str
    status: str
    description: str
    tools: List[str]

@dataclass
class ToolInfo:
    """工具信息"""
    name: str
    description: str
    parameters: Dict[str, Any]

class ContextType(Enum):
    """上下文类型"""
    STORE = "store"
    AGENT = "agent"

class MCPStoreContext:
    """
    MCPStore上下文类
    负责处理具体的业务操作，维护操作的上下文环境
    """
    def __init__(self, store: 'MCPStore', agent_id: Optional[str] = None):
        self._store = store
        self._agent_id = agent_id
        self._context_type = ContextType.STORE if agent_id is None else ContextType.AGENT
        
        # 扩展预留
        self._metadata: Dict[str, Any] = {}
        self._config: Dict[str, Any] = {}
        self._cache: Dict[str, Any] = {}

    # === 核心服务接口 ===
    async def list_services(self) -> List[ServiceInfo]:
        """
        列出服务列表
        - store上下文：聚合 main_client 下所有 client_id 的服务
        - agent上下文：聚合 agent_id 下所有 client_id 的服务
        """
        if self._context_type == ContextType.STORE:
            return await self._store.list_services()
        else:
            return await self._store.list_services(self._agent_id, agent_mode=True)

    async def add_service(self, config: Union[ServiceConfigUnion, List[str], None] = None) -> bool:
        """
        增强版的服务添加方法，支持多种配置格式：
        1. URL方式：
           await add_service({
               "name": "weather",
               "url": "https://weather-api.example.com/mcp",
               "transport": "streamable-http"
           })
        
        2. 本地命令方式：
           await add_service({
               "name": "assistant",
               "command": "python",
               "args": ["./assistant_server.py"],
               "env": {"DEBUG": "true"}
           })
        
        3. MCPConfig字典方式：
           await add_service({
               "mcpServers": {
                   "weather": {
                       "url": "https://weather-api.example.com/mcp"
                   }
               }
           })
        
        4. 服务名称列表方式（从现有配置中选择）：
           await add_service(['weather', 'assistant'])
        
        5. 无参数方式（仅限Store上下文）：
           await add_service()  # 注册所有服务
        
        所有新添加的服务都会同步到 mcp.json 配置文件中。
        """
        agent_id = self._agent_id or self._store.client_manager.main_client_id
        
        # 处理不同的输入格式
        if config is None:
            # Store模式下的全量注册
            if self._context_type == ContextType.STORE:
                print("[INFO][add_service] STORE模式-全量注册所有服务")
                resp = await self._store.register_json_service()
                print(f"[INFO][add_service] 注册结果: {resp}")
                return bool(resp and resp.service_names)
            else:
                print("[WARN][add_service] AGENT模式-未指定服务配置")
                return False
                
        # 处理服务名称列表
        if isinstance(config, list):
            if not config:
                print("[WARN][add_service] 服务名称列表为空")
                return False
                
            print(f"[INFO][add_service] 注册指定服务: {config}")
            resp = await self._store.register_json_service(
                client_id=agent_id,
                service_names=config
            )
            print(f"[INFO][add_service] 注册结果: {resp}")
            return bool(resp and resp.service_names)
            
        # 处理字典格式的配置
        if isinstance(config, dict):
            # 转换为标准格式
            if "mcpServers" in config:
                # 已经是MCPConfig格式
                mcp_config = config
            else:
                # 单个服务配置，需要转换为MCPConfig格式
                service_name = config.get("name")
                if not service_name:
                    print("[ERROR][add_service] 服务配置缺少name字段")
                    return False
                    
                mcp_config = {
                    "mcpServers": {
                        service_name: {k: v for k, v in config.items() if k != "name"}
                    }
                }
            
            # 更新配置文件
            try:
                # 1. 加载现有配置
                current_config = self._store.config.load_config()
                
                # 2. 合并新配置
                for name, service_config in mcp_config["mcpServers"].items():
                    current_config["mcpServers"][name] = service_config
                
                # 3. 保存更新后的配置
                self._store.config.save_config(current_config)
                
                # 4. 注册服务
                resp = await self._store.register_json_service(
                    client_id=agent_id,
                    service_names=list(mcp_config["mcpServers"].keys())
                )
                print(f"[INFO][add_service] 注册结果: {resp}")
                return bool(resp and resp.service_names)
                
            except Exception as e:
                print(f"[ERROR][add_service] 更新配置文件失败: {e}")
                return False
        
        print(f"[ERROR][add_service] 不支持的配置格式: {type(config)}")
        return False

    async def list_tools(self) -> List[ToolInfo]:
        """
        列出工具列表
        - store上下文：聚合 main_client 下所有 client_id 的工具
        - agent上下文：聚合 agent_id 下所有 client_id 的工具
        """
        if self._context_type == ContextType.STORE:
            return await self._store.list_tools()
        else:
            return await self._store.list_tools(self._agent_id, agent_mode=True)

    async def check_services(self) -> dict:
        """
        异步健康检查，store/agent上下文自动判断
        - store上下文：聚合 main_client 下所有 client_id 的服务健康状态
        - agent上下文：聚合 agent_id 下所有 client_id 的服务健康状态
        """
        if self._context_type.name == 'STORE':
            return await self._store.get_health_status()
        elif self._context_type.name == 'AGENT':
            return await self._store.get_health_status(self._agent_id, agent_mode=True)
        else:
            print(f"[ERROR][check_services] 未知上下文类型: {self._context_type}")
            return {}

    async def get_service_info(self, name: str) -> Any:
        """
        获取服务详情，支持 store/agent 上下文
        - store上下文：在 main_client 下的所有 client 中查找服务
        - agent上下文：在指定 agent_id 下的所有 client 中查找服务
        """
        if not name:
            return {}
            
        if self._context_type == ContextType.STORE:
            print(f"[INFO][get_service_info] STORE模式-在main_client中查找服务: {name}")
            return await self._store.get_service_info(name)
        elif self._context_type == ContextType.AGENT:
            print(f"[INFO][get_service_info] AGENT模式-在agent({self._agent_id})中查找服务: {name}")
            return await self._store.get_service_info(name, self._agent_id)
        else:
            print(f"[ERROR][get_service_info] 未知上下文类型: {self._context_type}")
            return {}

    async def use_tool(self, tool_name: str, args: Dict[str, Any]) -> Any:
        """
        使用工具，支持 store/agent 上下文
        - store上下文：在 main_client 下的所有 client 中查找并使用工具
        - agent上下文：在指定 agent_id 下的所有 client 中查找并使用工具
        
        Args:
            tool_name: 工具名称，格式为 service_toolname
            args: 工具参数
            
        Returns:
            Any: 工具执行结果
        """
        # 从工具名称中提取服务名称
        if "_" not in tool_name:
            raise ValueError(f"Invalid tool name format: {tool_name}. Expected format: service_toolname")
        
        if self._context_type == ContextType.STORE:
            print(f"[INFO][use_tool] STORE模式-在main_client中使用工具: {tool_name}")
            request = ToolExecutionRequest(
                tool_name=tool_name,
                args=args
            )
        else:
            print(f"[INFO][use_tool] AGENT模式-在agent({self._agent_id})中使用工具: {tool_name}")
            request = ToolExecutionRequest(
                tool_name=tool_name,
                args=args,
                agent_id=self._agent_id
            )
            
        return await self._store.process_tool_request(request)

    # === 上下文信息 ===
    @property
    def context_type(self) -> ContextType:
        """获取上下文类型"""
        return self._context_type

    @property
    def agent_id(self) -> Optional[str]:
        """获取当前agent_id"""
        return self._agent_id 



from fastapi import APIRouter, Depends, HTTPException
from typing import Dict, Any, Optional, List
from mcpstore.core.store import McpStore
from mcpstore.core.models.client import ClientRegistrationRequest, ClientRegistrationResponse
from mcpstore.scripts.deps import get_store
import logging

router = APIRouter()
logger = logging.getLogger(__name__)

@router.post("/register_clients", response_model=ClientRegistrationResponse)
async def register_clients(
    request: ClientRegistrationRequest,
    store: McpStore = Depends(get_store)
):
    """注册客户端"""
    return await store.register_clients(request) 



from typing import Dict, Any
from mcpstore.core.store import McpStore
from fastapi import HTTPException

# 全局应用状态
app_state: Dict[str, Any] = {}

def get_store() -> McpStore:
    store = app_state.get("store")
    if store is None:
        raise HTTPException(status_code=503, detail="Service not ready (Store not initialized)")
    return store



import uvicorn
import typer
import asyncio
import sys
from typing_extensions import Annotated
from mcpstore.scripts.app import app  # 导入 app 对象
import logging

# 导入独立运行模式
from mcpstore.cli.standalone import run_standalone

# Set up logging for the CLI itself
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
cli_logger = logging.getLogger("cli_main")


app_cli = typer.Typer(no_args_is_help=True)

@app_cli.callback()
def callback():
    """
    MCP Store Command Line Interface.
    """
    cli_logger.info("【第4步】Typer 回调函数已执行，准备分发子命令。")
    pass

@app_cli.command()
def api(
    host: Annotated[
        str, typer.Option(help="The host to bind to.")
    ] = "0.0.0.0",
    port: Annotated[
        int, typer.Option(help="The port to bind to.")
    ] = 18611,
    reload: Annotated[
        bool,
        typer.Option(
            help="Enable auto-reloading.",
        ),
    ] = False,
):
    """启动 mcpstore API 服务"""
    cli_logger.info(f"【第5步】Typer 已成功匹配到 'api' 命令。")
    cli_logger.info(f"    - 接收到参数 Host: {host}")
    cli_logger.info(f"    - 接收到参数 Port: {port}")
    cli_logger.info(f"    - 接收到参数 Reload: {reload}")
    cli_logger.info("【第6步】CLI 任务完成，准备将控制权移交给 Uvicorn。")
    uvicorn.run("mcpstore.scripts.app:app", host=host, port=port, reload=reload)

@app_cli.command()
def standalone(
    config: Annotated[
        str, typer.Option("--config", "-c", help="MCP配置文件路径")
    ] = None,
    debug: Annotated[
        bool, typer.Option("--debug", "-d", help="启用调试模式")
    ] = False,
    service: Annotated[
        str, typer.Option("--service", "-s", help="要连接的特定服务名称")
    ] = None,
    test_node: Annotated[
        str, typer.Option("--test-node", "-t", help="直接测试指定的Node.js脚本路径")
    ] = None,
):
    """在独立模式下运行MCP服务注册"""
    cli_logger.info(f"Typer 已成功匹配到 'standalone' 命令。")
    cli_logger.info(f"    - 接收到参数 Config: {config}")
    cli_logger.info(f"    - 接收到参数 Debug: {debug}")
    cli_logger.info(f"    - 接收到参数 Service: {service}")
    cli_logger.info(f"    - 接收到参数 Test Node: {test_node}")
    
    # 在Windows平台上设置事件循环策略
    if sys.platform == 'win32':
        asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
        cli_logger.info("已设置Windows ProactorEventLoop策略")
    
    # 运行独立模式
    exit_code = asyncio.run(run_standalone(
        config_path=config,
        debug=debug,
        service_name=service,
        test_node_path=test_node
    ))
    
    # 根据退出码退出
    if exit_code != 0:
        cli_logger.error("独立模式运行失败")
        sys.exit(exit_code)
    else:
        cli_logger.info("独立模式运行成功")

def main():
    cli_logger.info("【第3步】Typer 主应用已启动，准备解析命令行参数。")
    app_cli()

if __name__ == "__main__":
    cli_logger.info("【第1步】命令行入口 (__name__ == '__main__') 已触发。")
    cli_logger.info("【第2步】即将调用 main() 函数。")
    main() 



import os
import json
import random
import string
from datetime import datetime
from typing import Dict, Any, Optional, List
import logging

logger = logging.getLogger(__name__)

# 将所有配置文件统一放在 data/defaults 目录下
CLIENT_SERVICES_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'data', 'defaults', 'client_services.json')
AGENT_CLIENTS_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'data', 'defaults', 'agent_clients.json')

class ClientManager:
    """管理客户端配置的类"""
    
    def __init__(self, services_path: Optional[str] = None):
        """
        初始化客户端管理器
        
        Args:
            services_path: 配置文件目录
        """
        self.services_path = services_path or CLIENT_SERVICES_PATH
        self._ensure_file()
        self.client_services = self.load_all_clients()
        self.main_client_id = "main_client"  # 主客户端ID
        self._ensure_agent_clients_file()

    def _ensure_file(self):
        """确保客户端服务配置文件存在"""
        os.makedirs(os.path.dirname(self.services_path), exist_ok=True)
        if not os.path.exists(self.services_path):
            with open(self.services_path, 'w', encoding='utf-8') as f:
                json.dump({}, f)

    def _ensure_agent_clients_file(self):
        """确保agent-client映射文件存在"""
        os.makedirs(os.path.dirname(AGENT_CLIENTS_PATH), exist_ok=True)
        if not os.path.exists(AGENT_CLIENTS_PATH):
            with open(AGENT_CLIENTS_PATH, 'w', encoding='utf-8') as f:
                json.dump({}, f)

    def load_all_clients(self) -> Dict[str, Any]:
        """加载所有客户端配置"""
        with open(self.services_path, 'r', encoding='utf-8') as f:
            return json.load(f)

    def save_all_clients(self, data: Dict[str, Any]):
        """保存所有客户端配置"""
        with open(self.services_path, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
        # 更新内存中的数据
        self.client_services = data.copy()

    def get_client_config(self, client_id: str) -> Optional[Dict[str, Any]]:
        """获取客户端配置"""
        # 每次都重新加载以确保数据最新
        self.client_services = self.load_all_clients()
        return self.client_services.get(client_id)

    def save_client_config(self, client_id: str, config: Dict[str, Any]):
        """保存客户端配置"""
        all_clients = self.load_all_clients()
        all_clients[client_id] = config
        self.save_all_clients(all_clients)
        logger.info(f"Saved config for client_id={client_id}")

    def generate_client_id(self) -> str:
        """生成唯一的客户端ID"""
        ts = datetime.now().strftime("%Y%m%d%H%M%S")
        rand = ''.join(random.choices(string.ascii_lowercase + string.digits, k=6))
        return f"client_{ts}_{rand}"

    def create_client_config_from_names(self, service_names: List[str], mcp_config: Dict[str, Any]) -> Dict[str, Any]:
        """从服务名称列表生成新的客户端配置"""
        all_services = mcp_config.get("mcpServers", {})
        selected = {name: all_services[name] for name in service_names if name in all_services}
        return {"mcpServers": selected}

    def add_client(self, config: Dict[str, Any], client_id: Optional[str] = None) -> str:
        """
        添加新的客户端配置
        
        Args:
            config: 客户端配置
            client_id: 可选的客户端ID，如果不提供则自动生成
            
        Returns:
            使用的客户端ID
        """
        if not client_id:
            client_id = self.generate_client_id()
        self.client_services[client_id] = config
        self.save_client_config(client_id, config)
        return client_id
    
    def remove_client(self, client_id: str) -> bool:
        """
        移除客户端配置
        
        Args:
            client_id: 要移除的客户端ID
            
        Returns:
            是否成功移除
        """
        if client_id in self.client_services:
            del self.client_services[client_id]
            self.save_all_clients(self.client_services)
            return True
        return False
    
    def has_client(self, client_id: str) -> bool:
        """
        检查客户端是否存在
        
        Args:
            client_id: 客户端ID
            
        Returns:
            是否存在
        """
        # 每次检查都重新加载以确保数据最新
        self.client_services = self.load_all_clients()
        return client_id in self.client_services
    
    def get_all_clients(self) -> Dict[str, Any]:
        """
        获取所有客户端配置
        
        Returns:
            所有客户端配置的字典
        """
        # 每次获取都重新加载以确保数据最新
        self.client_services = self.load_all_clients()
        return self.client_services.copy()

    # === agent_clients.json 相关 ===
    def load_all_agent_clients(self) -> Dict[str, Any]:
        """加载所有agent-client映射"""
        self._ensure_agent_clients_file()
        with open(AGENT_CLIENTS_PATH, 'r', encoding='utf-8') as f:
            return json.load(f)

    def save_all_agent_clients(self, data: Dict[str, Any]):
        """保存agent-client映射"""
        with open(AGENT_CLIENTS_PATH, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)

    def get_agent_clients(self, agent_id: str) -> List[str]:
        """
        获取指定 agent 下的所有 client_id
        如果是 main_client，返回所有 client_id
        """
        if agent_id == self.main_client_id:
            return list(self.get_all_clients().keys())
        data = self.load_all_agent_clients()
        return data.get(agent_id, [])

    def add_agent_client_mapping(self, agent_id: str, client_id: str):
        """添加agent-client映射"""
        if agent_id == self.main_client_id:
            logger.debug("Skipping mapping for main_client as it's handled automatically")
            return
        data = self.load_all_agent_clients()
        if agent_id not in data:
            data[agent_id] = [client_id]
        elif client_id not in data[agent_id]:
            data[agent_id].append(client_id)
        self.save_all_agent_clients(data)
        logger.info(f"Mapped agent_id={agent_id} to client_id={client_id}")

    def remove_agent_client_mapping(self, agent_id: str, client_id: str):
        """移除agent-client映射"""
        if agent_id == self.main_client_id:
            logger.warning("Cannot remove mapping for main_client")
            return
        data = self.load_all_agent_clients()
        if agent_id in data and client_id in data[agent_id]:
            data[agent_id].remove(client_id)
            if not data[agent_id]:
                del data[agent_id]
            self.save_all_agent_clients(data)
            logger.info(f"Removed mapping agent_id={agent_id} to client_id={client_id}")

    def get_main_client_ids(self) -> List[str]:
        """获取 main_client 下的所有 client_id"""
        return list(self.get_all_clients().keys())

    def is_valid_client(self, client_id: str) -> bool:
        """检查是否是有效的 client_id"""
        return self.has_client(client_id) 



from fastapi import Request
from fastapi.responses import JSONResponse
from fastapi.exception_handlers import RequestValidationError

async def validation_exception_handler(request: Request, exc: RequestValidationError):
    return JSONResponse(
        status_code=422,
        content={"detail": exc.errors(), "body": exc.body},
    ) 



from fastapi import APIRouter, Depends, HTTPException, Path
from typing import Dict, Any, Optional, List
from mcpstore.core.store import McpStore
from mcpstore.core.models.service import (
    RegisterRequestUnion, JsonRegistrationResponse, JsonUpdateRequest, JsonConfigResponse,
    ServiceRegistrationResult, ServicesResponse, ServiceInfoResponse
)
from mcpstore.scripts.deps import get_store
import logging

router = APIRouter()
logger = logging.getLogger(__name__)

@router.post("/register", response_model=Dict[str, str])
async def register_service(
    payload: RegisterRequestUnion,
    agent_id: Optional[str] = None,
    store: McpStore = Depends(get_store)
):
    """注册服务"""
    return await store.register_service(payload, agent_id)

@router.post("/register/json", response_model=JsonRegistrationResponse)
async def register_json_service(
    client_id: Optional[str] = None,
    service_names: Optional[List[str]] = None,
    store: McpStore = Depends(get_store)
):
    """批量注册服务"""
    return await store.register_json_service(client_id, service_names)

@router.put("/register/json", response_model=JsonRegistrationResponse)
async def update_json_service(
    payload: JsonUpdateRequest,
    store: McpStore = Depends(get_store)
):
    """更新服务配置"""
    return await store.update_json_service(payload)

@router.get("/register/json", response_model=JsonConfigResponse)
async def get_json_config(
    client_id: Optional[str] = None,
    store: McpStore = Depends(get_store)
):
    """获取JSON配置
    
    Args:
        client_id: 客户端ID，若为 main_client 则查询主客户端配置，否则查询普通客户端配置
        
    Returns:
        JsonConfigResponse: 包含客户端配置的响应
    """
    try:
        return store.get_json_config(client_id)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Internal server error: {str(e)}")

@router.get("/health", response_model=Dict[str, Any])
async def get_health_status(store: McpStore = Depends(get_store)):
    """获取服务健康状态"""
    return await store.get_health_status()

@router.get("/service_info", response_model=ServiceInfoResponse)
async def get_service_info(name: str, store: McpStore = Depends(get_store)):
    """获取服务信息
    
    Args:
        name: 服务名称
        
    Returns:
        ServiceInfoResponse: 包含服务详细信息的响应
    """
    try:
        return await store.get_service_info(name)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))

@router.get("/services", response_model=ServicesResponse)
async def list_services(store: McpStore = Depends(get_store)):
    """获取所有服务列表"""
    return await store.list_services() 



import json
import logging
import os
import uuid
from datetime import datetime
from typing import List, Dict, Any, Optional

from fastapi import APIRouter, Depends, HTTPException
from pydantic import BaseModel

from mcpstore.scripts.deps import get_store
from mcpstore.core.store import McpStore
from mcpstore.scripts.models import ClientRegistrationRequest, ServiceRegistrationStatus, ClientRegistrationResponse

router = APIRouter()
logger = logging.getLogger(__name__)

def generate_agent_id() -> str:
    """生成agent_id"""
    timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
    random_suffix = str(uuid.uuid4())[:8]
    return f"agent_{timestamp}_{random_suffix}"

def generate_client_id(agent_id: str, index: int) -> str:
    """根据agent_id生成client_id"""
    timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
    return f"{agent_id}_client_{index}_{timestamp}"

def save_client_configs(configs: Dict[str, Any]):
    """保存客户端配置到文件"""
    config_dir = "configs"
    os.makedirs(config_dir, exist_ok=True)
    
    # 保存client_services.json
    client_services_path = os.path.join(config_dir, "client_services.json")
    with open(client_services_path, "w") as f:
        json.dump(configs, f, indent=2)

def save_agent_clients(agent_id: str, client_ids: List[str]):
    """保存agent和client的关系到文件"""
    config_dir = "configs"
    agent_clients_path = os.path.join(config_dir, "agent_clients.json")
    
    # 读取现有配置
    agent_clients = {}
    if os.path.exists(agent_clients_path):
        with open(agent_clients_path, "r") as f:
            agent_clients = json.load(f)
    
    # 更新配置
    agent_clients[agent_id] = client_ids
    
    # 保存配置
    with open(agent_clients_path, "w") as f:
        json.dump(agent_clients, f, indent=2)

def load_client_configs() -> Dict[str, Any]:
    """加载客户端配置"""
    config_dir = "configs"
    client_services_path = os.path.join(config_dir, "client_services.json")
    
    if os.path.exists(client_services_path):
        with open(client_services_path, "r") as f:
            return json.load(f)
    return {}

def load_agent_clients() -> Dict[str, List[str]]:
    """加载agent和client的关系"""
    config_dir = "configs"
    agent_clients_path = os.path.join(config_dir, "agent_clients.json")
    
    if os.path.exists(agent_clients_path):
        with open(agent_clients_path, "r") as f:
            return json.load(f)
    return {}

@router.post("/register_clients", response_model=ClientRegistrationResponse)
async def register_clients(client_configs: Dict[str, Any], store: McpStore = Depends(get_store)):
    try:
        return store.register_clients(client_configs)
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e)) 



from mcpstore.core.orchestrator import MCPOrchestrator
from mcpstore.core.registry import ServiceRegistry
from mcpstore.plugins.json_mcp import MCPConfig
from mcpstore.core.client_manager import ClientManager
from mcpstore.core.session_manager import SessionManager
from mcpstore.core.models.service import (
    RegisterRequestUnion, JsonRegistrationResponse, JsonUpdateRequest, JsonConfigResponse,
    ServiceInfo, ServicesResponse, TransportType, ServiceInfoResponse,
    ServiceRegistrationResult
)
from mcpstore.core.models.client import ClientRegistrationResponse
from mcpstore.core.models.tool import (
    ToolExecutionResponse, ToolInfo, ToolsResponse, ToolExecutionRequest
)
import logging
from typing import Optional, List, Dict, Any, Union
from .context import MCPStoreContext

logger = logging.getLogger(__name__)

class MCPStore:
    """
    MCPStore - 智能体工具服务商店
    提供上下文切换的入口和通用操作
    """
    def __init__(self, orchestrator: MCPOrchestrator, config: MCPConfig):
        self.orchestrator = orchestrator
        self.config = config
        self.registry = orchestrator.registry
        self.client_manager = orchestrator.client_manager
        self.session_manager = orchestrator.session_manager
        self.logger = logging.getLogger(__name__)
        self._context_cache: Dict[str, MCPStoreContext] = {}
        self._store_context = self._create_store_context()

    def _create_store_context(self) -> MCPStoreContext:
        """创建商店级别的上下文"""
        return MCPStoreContext(self)

    @staticmethod
    def setup_store():
        config = MCPConfig()
        registry = ServiceRegistry()
        orchestrator = MCPOrchestrator(config.load_config(), registry)
        return MCPStore(orchestrator, config)
  
    def _create_agent_context(self, agent_id: str) -> MCPStoreContext:
        """创建agent级别的上下文"""
        return MCPStoreContext(self, agent_id)

    def for_store(self) -> MCPStoreContext:
        """获取商店级别的上下文"""
        # main_client 作为 store agent_id
        return self._store_context

    def for_agent(self, agent_id: str) -> MCPStoreContext:
        """获取agent级别的上下文（带缓存）"""
        if agent_id not in self._context_cache:
            self._context_cache[agent_id] = self._create_agent_context(agent_id)
        return self._context_cache[agent_id]

    async def register_service(self, payload: RegisterRequestUnion, agent_id: Optional[str] = None) -> Dict[str, str]:
        """重构：注册服务，支持批量 service_names 注册"""
        service_names = getattr(payload, 'service_names', None)
        if not service_names:
            raise ValueError("payload 必须包含 service_names 字段")
        results = {}
        agent_key = agent_id or self.client_manager.main_client_id
        for name in service_names:
            success, msg = await self.orchestrator.connect_service(name)
            if not success:
                results[name] = f"连接失败: {msg}"
                continue
            session = self.registry.get_session(agent_key, name)
            if not session:
                results[name] = "未能获取 session"
                continue
            tools = []
            try:
                tools = await session.list_tools() if hasattr(session, 'list_tools') else []
            except Exception as e:
                results[name] = f"获取工具失败: {e}"
                continue
            added_tools = self.registry.add_service(agent_key, name, session, [(tool['name'], tool) for tool in tools])
            results[name] = f"注册成功，工具数: {len(added_tools)}"
        return results

    async def register_json_service(self, client_id: Optional[str] = None, service_names: Optional[List[str]] = None) -> JsonRegistrationResponse:
        """批量注册服务，等价于 /register/json"""
        # 情况1: main_client 视为特殊 agent，注册所有 mcp.json 服务
        if client_id and client_id == self.client_manager.main_client_id and not service_names:
            all_services = self.config.load_config().get("mcpServers", {})
            agent_id = self.client_manager.main_client_id
            registered_client_ids = []
            registered_services = []
            for name in all_services.keys():
                new_client_id = self.client_manager.generate_client_id()
                client_config = {"mcpServers": {name: all_services[name]}}
                self.client_manager.save_client_config(new_client_id, client_config)
                self.client_manager.add_agent_client_mapping(agent_id, new_client_id)
                await self.orchestrator.register_json_services(client_config, client_id=new_client_id)
                registered_client_ids.append(new_client_id)
                registered_services.append(name)
            return JsonRegistrationResponse(
                client_id=agent_id,
                service_names=registered_services,
                config={"client_ids": registered_client_ids, "services": registered_services}
            )
        # 情况2: 没有传入client_id但传入了service_names
        elif not client_id and service_names:
            config = self.orchestrator.create_client_config_from_names(service_names)
            import time; agent_id = f"agent_{int(time.time() * 1000)}"
            results = await self.orchestrator.register_json_services(config)
            return JsonRegistrationResponse(
                client_id=agent_id,
                service_names=list(results.get("services", {}).keys()),
                config=config
            )
        # 情况3: 既没有传入client_id也没有传入service_names
        elif not client_id and not service_names:
            # 默认等价于 main_client 全量注册
            return await self.register_json_service(client_id=self.client_manager.main_client_id)
        # 情况4: agent级别多服务注册，强隔离
        else:
            all_services = self.config.load_config().get("mcpServers", {})
            agent_id = client_id
            registered_client_ids = []
            registered_services = []
            for name in service_names or []:
                if name not in all_services:
                    print(f"[WARN][register_json_service] 服务 {name} 未在全局配置中找到，跳过")
                    continue
                new_client_id = self.client_manager.generate_client_id()
                client_config = {"mcpServers": {name: all_services[name]}}
                self.client_manager.save_client_config(new_client_id, client_config)
                self.client_manager.add_agent_client_mapping(agent_id, new_client_id)
                await self.orchestrator.register_json_services(client_config, client_id=new_client_id)
                registered_client_ids.append(new_client_id)
                registered_services.append(name)
            return JsonRegistrationResponse(
                client_id=agent_id,
                service_names=registered_services,
                config={"client_ids": registered_client_ids, "services": registered_services}
            )

    async def update_json_service(self, payload: JsonUpdateRequest) -> JsonRegistrationResponse:
        """更新服务配置，等价于 PUT /register/json"""
        results = await self.orchestrator.register_json_services(
            config=payload.config,
            client_id=payload.client_id
        )
        return JsonRegistrationResponse(
            client_id=results.get("client_id", payload.client_id or "main_client"),
            service_names=list(results.get("services", {}).keys()),
            config=payload.config
        )

    def get_json_config(self, client_id: Optional[str] = None) -> JsonConfigResponse:
        """查询服务配置，等价于 GET /register/json"""
        if not client_id or client_id == self.client_manager.main_client_id:
            config = self.config.load_config()
            return JsonConfigResponse(
                client_id=self.client_manager.main_client_id,
                config=config
            )
        else:
            config = self.client_manager.get_client_config(client_id)
            if not config:
                raise ValueError(f"Client configuration not found: {client_id}")
            return JsonConfigResponse(
                client_id=client_id,
                config=config
            )

    async def process_tool_request(self, request: ToolExecutionRequest) -> ToolExecutionResponse:
        """
        处理工具执行请求
        - 验证工具名称格式
        - 转发请求到 orchestrator 执行
        
        Args:
            request: 工具执行请求
            
        Returns:
            ToolExecutionResponse: 工具执行响应
        """
        try:
            # 从工具名称中提取服务名称
            if "_" not in request.tool_name:
                raise ValueError(f"Invalid tool name format: {request.tool_name}. Expected format: service_toolname")
            
            service_name = request.tool_name.split("_")[0]
            
            # 执行工具
            result = await self.orchestrator.execute_tool(
                service_name=service_name,
                tool_name=request.tool_name,
                parameters=request.args,
                agent_id=request.agent_id
            )
            
            return ToolExecutionResponse(
                success=True,
                result=result
            )
        except Exception as e:
            logger.error(f"Tool execution failed: {e}")
            return ToolExecutionResponse(
                success=False,
                error=str(e)
            )

    def register_clients(self, client_configs: Dict[str, Any]) -> ClientRegistrationResponse:
        """注册客户端，等价于 /register_clients"""
        # 这里只是示例，具体实现需根据 client_manager 逻辑完善
        for client_id, config in client_configs.items():
            self.client_manager.save_client_config(client_id, config)
        return ClientRegistrationResponse(status="success", client_ids=list(client_configs.keys()))

    async def get_health_status(self, id: Optional[str] = None, agent_mode: bool = False) -> Dict[str, Any]:
        """
        获取服务健康状态：
        - store未传id 或 id==main_client：聚合 main_client 下所有 client_id 的服务健康状态
        - store传普通 client_id：只查该 client_id 下的服务健康状态
        - agent级别：聚合 agent_id 下所有 client_id 的服务健康状态；如果 id 不是 agent_id，尝试作为 client_id 查
        """
        from mcpstore.core.client_manager import ClientManager
        client_manager: ClientManager = self.client_manager
        services = []
        # 1. store未传id 或 id==main_client，聚合 main_client 下所有 client_id 的服务健康状态
        if not agent_mode and (not id or id == self.client_manager.main_client_id):
            client_ids = client_manager.get_agent_clients(self.client_manager.main_client_id)
            for client_id in client_ids:
                service_names = self.registry.get_all_service_names(client_id)
                for name in service_names:
                    config = self.config.get_service_config(name) or {}
                    is_healthy = await self.orchestrator.is_service_healthy(name, client_id)
                    service_status = {
                        "name": name,
                        "url": config.get("url", ""),
                        "transport_type": config.get("transport", ""),
                        "status": "healthy" if is_healthy else "unhealthy",
                        "command": config.get("command"),
                        "args": config.get("args"),
                        "package_name": config.get("package_name")
                    }
                    services.append(service_status)
            return {
                "orchestrator_status": "running",
                "active_services": len(services),
                "services": services
            }
        # 2. store传普通 client_id，只查该 client_id 下的服务健康状态
        if not agent_mode and id:
            if id == self.client_manager.main_client_id:
                return {
                    "orchestrator_status": "running",
                    "active_services": 0,
                    "services": []
                }
            service_names = self.registry.get_all_service_names(id)
            for name in service_names:
                config = self.config.get_service_config(name) or {}
                is_healthy = await self.orchestrator.is_service_healthy(name, id)
                service_status = {
                    "name": name,
                    "url": config.get("url", ""),
                    "transport_type": config.get("transport", ""),
                    "status": "healthy" if is_healthy else "unhealthy",
                    "command": config.get("command"),
                    "args": config.get("args"),
                    "package_name": config.get("package_name")
                }
                services.append(service_status)
            return {
                "orchestrator_status": "running",
                "active_services": len(services),
                "services": services
            }
        # 3. agent级别，聚合 agent_id 下所有 client_id 的服务健康状态；如果 id 不是 agent_id，尝试作为 client_id 查
        if agent_mode and id:
            client_ids = client_manager.get_agent_clients(id)
            if client_ids:
                for client_id in client_ids:
                    service_names = self.registry.get_all_service_names(client_id)
                    for name in service_names:
                        config = self.config.get_service_config(name) or {}
                        is_healthy = await self.orchestrator.is_service_healthy(name, client_id)
                        service_status = {
                            "name": name,
                            "url": config.get("url", ""),
                            "transport_type": config.get("transport", ""),
                            "status": "healthy" if is_healthy else "unhealthy",
                            "command": config.get("command"),
                            "args": config.get("args"),
                            "package_name": config.get("package_name")
                        }
                        services.append(service_status)
                return {
                    "orchestrator_status": "running",
                    "active_services": len(services),
                    "services": services
                }
            else:
                service_names = self.registry.get_all_service_names(id)
                for name in service_names:
                    config = self.config.get_service_config(name) or {}
                    is_healthy = await self.orchestrator.is_service_healthy(name, id)
                    service_status = {
                        "name": name,
                        "url": config.get("url", ""),
                        "transport_type": config.get("transport", ""),
                        "status": "healthy" if is_healthy else "unhealthy",
                        "command": config.get("command"),
                        "args": config.get("args"),
                        "package_name": config.get("package_name")
                    }
                    services.append(service_status)
                return {
                    "orchestrator_status": "running",
                    "active_services": len(services),
                    "services": services
                }
        return {
            "orchestrator_status": "running",
            "active_services": 0,
            "services": []
        }

    async def get_service_info(self, name: str, agent_id: Optional[str] = None) -> ServiceInfoResponse:
        """
        获取服务详细信息：
        - 未传 agent_id：在 main_client 下所有 client_id 中查找服务
        - 传 agent_id：在该 agent_id 下所有 client_id 中查找服务
        """
        from mcpstore.core.client_manager import ClientManager
        client_manager: ClientManager = self.client_manager

        # 获取要查找的 client_ids
        if not agent_id:
            client_ids = client_manager.get_agent_clients(self.client_manager.main_client_id)
        else:
            client_ids = client_manager.get_agent_clients(agent_id)

        # 在所有相关的 client 中查找服务
        for client_id in client_ids:
            if self.registry.has_service(client_id, name):
                # 获取服务配置
                config = self.config.get_service_config(name) or {}
                service_tools = self.registry.get_tools_for_service(client_id, name)
                
                # 获取工具详细信息
                detailed_tools = []
                for tool_name in service_tools:
                    tool_info = self.registry._get_detailed_tool_info(client_id, tool_name)
                    if tool_info:
                        detailed_tools.append(tool_info)
                
                # 获取服务健康状态
                is_healthy = await self.orchestrator.is_service_healthy(name, client_id)
                
                # 构建服务信息
                service_info = ServiceInfo(
                    url=config.get("url", ""),
                    name=name,
                    transport_type=self._infer_transport_type(config),
                    status="healthy" if is_healthy else "unhealthy",
                    tool_count=len(service_tools),
                    keep_alive=config.get("keep_alive", False),
                    working_dir=config.get("working_dir"),
                    env=config.get("env"),
                    last_heartbeat=self.registry.get_last_heartbeat(client_id, name),
                    command=config.get("command"),
                    args=config.get("args"),
                    package_name=config.get("package_name")
                )
                
                return ServiceInfoResponse(
                    service=service_info,
                    tools=detailed_tools,
                    connected=True
                )

        return ServiceInfoResponse(
            service=None,
            tools=[],
            connected=False
        )

    def _infer_transport_type(self, service_config: Dict[str, Any]) -> TransportType:
        """推断服务的传输类型"""
        if not service_config:
            return TransportType.STREAMABLE_HTTP
            
        # 优先使用 transport 字段
        transport = service_config.get("transport")
        if transport:
            try:
                return TransportType(transport)
            except ValueError:
                pass
                
        # 其次根据 url 判断
        if service_config.get("url"):
            return TransportType.STREAMABLE_HTTP
            
        # 根据 command/args 判断
        cmd = (service_config.get("command") or "").lower()
        args = " ".join(service_config.get("args", [])).lower()
        
        if "python" in cmd or ".py" in args:
            return TransportType.STDIO_PYTHON
        if "node" in cmd or ".js" in args:
            return TransportType.STDIO_NODE
        if "uvx" in cmd:
            return TransportType.STDIO  # 使用通用的STDIO类型
        if "npx" in cmd:
            return TransportType.STDIO  # 使用通用的STDIO类型
            
        return TransportType.STREAMABLE_HTTP

    async def list_services(self, id: Optional[str] = None, agent_mode: bool = False) -> List[ServiceInfo]:
        """
        获取服务列表：
        - store未传id 或 id==main_client：聚合 main_client 下所有 client_id 的服务
        - store传普通 client_id：只查该 client_id 下的服务
        - agent级别：聚合 agent_id 下所有 client_id 的服务；如果 id 不是 agent_id，尝试作为 client_id 查
        """
        from mcpstore.core.client_manager import ClientManager
        client_manager: ClientManager = self.client_manager
        services_info = []
        # 1. store未传id 或 id==main_client，聚合 main_client 下所有 client_id 的服务
        if not agent_mode and (not id or id == self.client_manager.main_client_id):
            client_ids = client_manager.get_agent_clients(self.client_manager.main_client_id)
            for client_id in client_ids:
                service_names = self.registry.get_all_service_names(client_id)
                for name in service_names:
                    details = self.registry.get_service_details(client_id, name)
                    config = self.config.get_service_config(name) or {}
                    is_healthy = await self.orchestrator.is_service_healthy(name, client_id)
                    service_info = ServiceInfo(
                        url=config.get("url", ""),
                        name=name,
                        transport_type=self._infer_transport_type(config),
                        status="healthy" if is_healthy else "unhealthy",
                        tool_count=details.get("tool_count", 0),
                        keep_alive=config.get("keep_alive", False),
                        working_dir=config.get("working_dir"),
                        env=config.get("env"),
                        last_heartbeat=self.registry.get_last_heartbeat(client_id, name),
                        command=config.get("command"),
                        args=config.get("args"),
                        package_name=config.get("package_name")
                    )
                    services_info.append(service_info)
            return services_info
        # 2. store传普通 client_id，只查该 client_id 下的服务
        if not agent_mode and id:
            if id == self.client_manager.main_client_id:
                # 已在上面聚合分支处理，这里直接返回空
                return services_info
            service_names = self.registry.get_all_service_names(id)
            for name in service_names:
                details = self.registry.get_service_details(id, name)
                config = self.config.get_service_config(name) or {}
                is_healthy = await self.orchestrator.is_service_healthy(name, id)
                service_info = ServiceInfo(
                    url=config.get("url", ""),
                    name=name,
                    transport_type=self._infer_transport_type(config),
                    status="healthy" if is_healthy else "unhealthy",
                    tool_count=details.get("tool_count", 0),
                    keep_alive=config.get("keep_alive", False),
                    working_dir=config.get("working_dir"),
                    env=config.get("env"),
                    last_heartbeat=self.registry.get_last_heartbeat(id, name),
                    command=config.get("command"),
                    args=config.get("args"),
                    package_name=config.get("package_name")
                )
                services_info.append(service_info)
            return services_info
        # 3. agent级别，聚合 agent_id 下所有 client_id 的服务；如果 id 不是 agent_id，尝试作为 client_id 查
        if agent_mode and id:
            client_ids = client_manager.get_agent_clients(id)
            if client_ids:
                for client_id in client_ids:
                    service_names = self.registry.get_all_service_names(client_id)
                    for name in service_names:
                        details = self.registry.get_service_details(client_id, name)
                        config = self.config.get_service_config(name) or {}
                        is_healthy = await self.orchestrator.is_service_healthy(name, client_id)
                        service_info = ServiceInfo(
                            url=config.get("url", ""),
                            name=name,
                            transport_type=self._infer_transport_type(config),
                            status="healthy" if is_healthy else "unhealthy",
                            tool_count=details.get("tool_count", 0),
                            keep_alive=config.get("keep_alive", False),
                            working_dir=config.get("working_dir"),
                            env=config.get("env"),
                            last_heartbeat=self.registry.get_last_heartbeat(client_id, name),
                            command=config.get("command"),
                            args=config.get("args"),
                            package_name=config.get("package_name")
                        )
                        services_info.append(service_info)
                return services_info
            else:
                service_names = self.registry.get_all_service_names(id)
                for name in service_names:
                    details = self.registry.get_service_details(id, name)
                    config = self.config.get_service_config(name) or {}
                    is_healthy = await self.orchestrator.is_service_healthy(name, id)
                    service_info = ServiceInfo(
                        url=config.get("url", ""),
                        name=name,
                        transport_type=self._infer_transport_type(config),
                        status="healthy" if is_healthy else "unhealthy",
                        tool_count=details.get("tool_count", 0),
                        keep_alive=config.get("keep_alive", False),
                        working_dir=config.get("working_dir"),
                        env=config.get("env"),
                        last_heartbeat=self.registry.get_last_heartbeat(id, name),
                        command=config.get("command"),
                        args=config.get("args"),
                        package_name=config.get("package_name")
                    )
                    services_info.append(service_info)
                return services_info
        return services_info

    async def list_tools(self, id: Optional[str] = None, agent_mode: bool = False) -> List[ToolInfo]:
        """
        列出工具列表：
        - store未传id 或 id==main_client：聚合 main_client 下所有 client_id 的工具
        - store传普通 client_id：只查该 client_id 下的工具
        - agent级别：聚合 agent_id 下所有 client_id 的工具；如果 id 不是 agent_id，尝试作为 client_id 查
        """
        from mcpstore.core.client_manager import ClientManager
        client_manager: ClientManager = self.client_manager
        tools = []
        # 1. store未传id 或 id==main_client，聚合 main_client 下所有 client_id 的工具
        if not agent_mode and (not id or id == self.client_manager.main_client_id):
            client_ids = client_manager.get_agent_clients(self.client_manager.main_client_id)
            for client_id in client_ids:
                tool_dicts = self.registry.get_all_tool_info(client_id)
                for tool in tool_dicts:
                    tools.append(ToolInfo(
                        name=tool.get("name", ""),
                        description=tool.get("description", ""),
                        service_name=tool.get("service_name", ""),
                        client_id=tool.get("client_id", ""),
                        inputSchema=tool.get("inputSchema", {})
                    ))
            return tools
        # 2. store传普通 client_id，只查该 client_id 下的工具
        if not agent_mode and id:
            if id == self.client_manager.main_client_id:
                return tools
            tool_dicts = self.registry.get_all_tool_info(id)
            for tool in tool_dicts:
                tools.append(ToolInfo(
                    name=tool.get("name", ""),
                    description=tool.get("description", ""),
                    service_name=tool.get("service_name", ""),
                    client_id=tool.get("client_id", ""),
                    inputSchema=tool.get("inputSchema", {})
                ))
            return tools
        # 3. agent级别，聚合 agent_id 下所有 client_id 的工具；如果 id 不是 agent_id，尝试作为 client_id 查
        if agent_mode and id:
            client_ids = client_manager.get_agent_clients(id)
            if client_ids:
                for client_id in client_ids:
                    tool_dicts = self.registry.get_all_tool_info(client_id)
                    for tool in tool_dicts:
                        tools.append(ToolInfo(
                            name=tool.get("name", ""),
                            description=tool.get("description", ""),
                            service_name=tool.get("service_name", ""),
                            client_id=tool.get("client_id", ""),
                            inputSchema=tool.get("inputSchema", {})
                        ))
                return tools
            else:
                tool_dicts = self.registry.get_all_tool_info(id)
                for tool in tool_dicts:
                    tools.append(ToolInfo(
                        name=tool.get("name", ""),
                        description=tool.get("description", ""),
                        service_name=tool.get("service_name", ""),
                        client_id=tool.get("client_id", ""),
                        inputSchema=tool.get("inputSchema", {})
                    ))
                return tools
        return tools

    async def use_tool(self, tool_name: str, args: Dict[str, Any]) -> Any:
        """
        使用工具（通用接口）
        
        Args:
            tool_name: 工具名称，格式为 service_toolname
            args: 工具参数
            
        Returns:
            Any: 工具执行结果
        """
        from mcpstore.core.models.tool import ToolExecutionRequest
        
        # 构造请求
        request = ToolExecutionRequest(
            tool_name=tool_name,
            args=args
        )
        
        # 处理工具请求
        return await self.process_tool_request(request)

    async def _add_service(self, service_names: List[str], agent_id: Optional[str]) -> bool:
        """内部方法：批量添加服务，store级别支持全量注册，agent级别支持指定服务注册"""
        # store级别
        if agent_id is None:
            if not service_names:
                # 全量注册
                resp = await self.register_json_service()
                return bool(resp and resp.service_names)
            else:
                self.logger.warning("Store级别暂不支持单独添加服务")
                return False
        # agent级别
        else:
            if service_names:
                resp = await self.register_json_service(client_id=agent_id, service_names=service_names)
                return bool(resp and resp.service_names)
            else:
                self.logger.warning("Agent级别添加服务时必须指定service_names")
                return False

    async def add_service(self, service_names: List[str], agent_id: Optional[str] = None) -> bool:
        context = self.for_agent(agent_id) if agent_id else self.for_store()
        return await context.add_service(service_names)

    def check_services(self, agent_id: Optional[str] = None) -> Dict[str, str]:
        """兼容旧版API"""
        context = self.for_agent(agent_id) if agent_id else self.for_store()
        return context.check_services() 



from pydantic import BaseModel, Field
from typing import Optional, List, Dict, Any, Literal, Union
from enum import Enum
from datetime import datetime

class TransportType(str, Enum):
    STREAMABLE_HTTP = "streamable_http"
    STDIO = "stdio"
    STDIO_PYTHON = "stdio_python"
    STDIO_NODE = "stdio_node"
    STDIO_SHELL = "stdio_shell"

class ServiceInfo(BaseModel):
    url: str = ""
    name: str
    transport_type: TransportType
    status: Literal["healthy", "unhealthy"]
    tool_count: int
    keep_alive: bool
    working_dir: Optional[str] = None
    env: Optional[Dict[str, str]] = None
    last_heartbeat: Optional[datetime] = None
    command: Optional[str] = None
    args: Optional[List[str]] = None
    package_name: Optional[str] = None

class ServiceInfoResponse(BaseModel):
    """单个服务的详细信息响应模型"""
    service: ServiceInfo
    tools: List[Dict[str, Any]]
    connected: bool

class ServicesResponse(BaseModel):
    services: List[ServiceInfo]
    total_services: int
    total_tools: int

class RegisterRequestUnion(BaseModel):
    url: Optional[str] = None
    name: Optional[str] = None
    transport: Optional[str] = None
    keep_alive: Optional[bool] = None
    working_dir: Optional[str] = None
    env: Optional[Dict[str, str]] = None
    command: Optional[str] = None
    args: Optional[List[str]] = None
    package_name: Optional[str] = None

class JsonUpdateRequest(BaseModel):
    client_id: Optional[str] = None
    service_names: Optional[List[str]] = None
    config: Dict[str, Any]

class JsonRegistrationResponse(BaseModel):
    client_id: str
    service_names: List[str]
    config: Dict[str, Any]

class JsonConfigResponse(BaseModel):
    client_id: str
    config: Dict[str, Any]

class ServiceRegistrationResult(BaseModel):
    success: bool
    message: str

class ServiceConfig(BaseModel):
    """服务配置基类"""
    name: str = Field(..., description="服务名称")

class URLServiceConfig(ServiceConfig):
    """URL方式的服务配置"""
    url: str = Field(..., description="服务URL")
    transport: Optional[str] = Field("streamable-http", description="传输类型: streamable-http 或 sse")
    headers: Optional[Dict[str, str]] = Field(default=None, description="请求头")

class CommandServiceConfig(ServiceConfig):
    """本地命令方式的服务配置"""
    command: str = Field(..., description="执行命令")
    args: Optional[List[str]] = Field(default=None, description="命令参数")
    env: Optional[Dict[str, str]] = Field(default=None, description="环境变量")
    working_dir: Optional[str] = Field(default=None, description="工作目录")

class MCPServerConfig(BaseModel):
    """完整的MCP服务配置"""
    mcpServers: Dict[str, Dict[str, Any]] = Field(..., description="MCP服务配置字典")

# 支持多种配置格式
ServiceConfigUnion = Union[URLServiceConfig, CommandServiceConfig, MCPServerConfig, Dict[str, Any]]

class AddServiceRequest(BaseModel):
    """添加服务请求"""
    config: ServiceConfigUnion = Field(..., description="服务配置，支持多种格式")
    update_config: bool = Field(default=True, description="是否更新配置文件") 



from enum import Enum
from typing import Dict, Any, Optional, List, Literal, Union
from pydantic import BaseModel, Field, HttpUrl, validator
import os
from datetime import datetime

class TransportType(str, Enum):
    """传输类型枚举，与 FastMCP 的传输类型名称匹配"""
    STREAMABLE_HTTP = "streamable-http"
    SSE = "sse"
    PYTHON_STDIO = "python-stdio"
    NODE_STDIO = "node-stdio"
    UVX_STDIO = "uvx-stdio"
    NPX_STDIO = "npx-stdio"

class BaseRegisterRequest(BaseModel):
    """注册请求的基础模型"""
    name: str = ""  # 服务名称，如果为空则自动生成
    env: Optional[Dict[str, str]] = None  # 环境变量配置
    keep_alive: bool = True  # 会话持久化选项

    @validator('name')
    def validate_name(cls, v):
        if v and not v.replace('_', '').replace('-', '').isalnum():
            raise ValueError("Service name can only contain letters, numbers, underscores and hyphens")
        return v

    @validator('env')
    def validate_env(cls, v):
        if v is not None:
            # 验证环境变量名称格式
            for key in v.keys():
                if not key.replace('_', '').isalnum():
                    raise ValueError(f"Invalid environment variable name: {key}")
        return v

class HTTPRegisterRequest(BaseRegisterRequest):
    """HTTP/SSE 服务注册请求模型"""
    url: HttpUrl  # 服务URL
    transport_type: Literal[TransportType.STREAMABLE_HTTP, TransportType.SSE] = TransportType.STREAMABLE_HTTP
    headers: Optional[Dict[str, str]] = None  # 自定义请求头

class StdioRegisterRequest(BaseRegisterRequest):
    """本地脚本服务注册请求模型"""
    script_path: str  # 脚本路径
    command: Optional[str] = None  # 可选的命令
    args: Optional[List[str]] = None  # 命令行参数
    transport_type: Literal[TransportType.PYTHON_STDIO, TransportType.NODE_STDIO]
    working_dir: Optional[str] = None  # 工作目录
    
    @validator('script_path')
    def validate_script_path(cls, v, values):
        if not v.endswith(('.py', '.js')):
            raise ValueError("Script path must end with .py or .js")
        
        # 验证文件是否存在
        if not os.path.exists(v):
            raise ValueError(f"Script file not found: {v}")
            
        # 验证文件扩展名与传输类型匹配
        if values.get('transport_type') == TransportType.PYTHON_STDIO and not v.endswith('.py'):
            raise ValueError("Python transport requires a .py file")
        if values.get('transport_type') == TransportType.NODE_STDIO and not v.endswith('.js'):
            raise ValueError("Node transport requires a .js file")
            
        return v

class ToolRegisterRequest(BaseRegisterRequest):
    """工具服务注册请求模型"""
    tool_name: str  # 工具名称
    args: Optional[List[str]] = None  # 工具参数
    transport_type: Literal[TransportType.UVX_STDIO, TransportType.NPX_STDIO]
    package_name: Optional[str] = None  # NPX 包名

# 用于统一接口的联合类型
RegisterRequestUnion = Union[HTTPRegisterRequest, StdioRegisterRequest, ToolRegisterRequest]




class ToolInfo(BaseModel):
    """工具信息"""
    name: str
    description: str
    service_name: Optional[str] = None
    client_id: Optional[str] = None  # 关联的client_id
    inputSchema: Optional[Dict[str, Any]] = None

class ServiceInfo(BaseModel):
    """服务信息模型"""
    url: str = ""  # URL 默认为空字符串
    name: str
    transport_type: TransportType
    status: Literal["healthy", "unhealthy"]
    tool_count: int
    keep_alive: bool
    working_dir: Optional[str] = None
    env: Optional[Dict[str, str]] = None
    last_heartbeat: Optional[datetime] = None
    command: Optional[str] = None  # 命令（用于 STDIO 服务）
    args: Optional[List[str]] = None  # 参数列表（用于 STDIO 服务）
    package_name: Optional[str] = None  # NPX 包名（用于 NPX 服务）

class ServicesResponse(BaseModel):
    """服务列表响应模型"""
    services: List[ServiceInfo]
    total_services: int
    total_tools: int
    
class ToolsResponse(BaseModel):
    """工具列表响应"""
    tools: List[ToolInfo]
    total_tools: int 

class JsonRegisterRequest(BaseModel):
    """JSON注册请求模型"""
    config: Dict[str, Any] = Field(..., description="MCP服务配置字典")
    client_id: Optional[str] = Field(None, description="客户端ID，如果不提供则自动生成")

class ServiceRegistrationResult(BaseModel):
    """服务注册结果模型"""
    status: Literal["success", "failed"]
    message: str
    error: Optional[str] = None

class JsonRegistrationResponse(BaseModel):
    """JSON注册响应模型"""
    client_id: str
    services: Dict[str, ServiceRegistrationResult]
    total_success: int
    total_failed: int

class JsonUpdateRequest(BaseModel):
    """JSON更新请求模型"""
    config: Dict[str, Any] = Field(..., description="完整的MCP服务配置字典")
    client_id: Optional[str] = Field(None, description="客户端ID，默认使用main_client")

class JsonConfigResponse(BaseModel):
    """JSON配置响应模型"""
    client_id: str
    config: Dict[str, Any]

class ContentType(str, Enum):
    """内容类型枚举"""
    TEXT = "text"
    IMAGE = "image"
    JSON = "json"
    BINARY = "binary"

class Content(BaseModel):
    """基础内容模型"""
    type: ContentType
    mime_type: Optional[str] = None

class TextContent(Content):
    """文本内容"""
    type: ContentType = ContentType.TEXT
    text: str

class ImageContent(Content):
    """图像内容"""
    type: ContentType = ContentType.IMAGE
    url: str
    alt: Optional[str] = None

class JsonContent(Content):
    """JSON内容"""
    type: ContentType = ContentType.JSON
    data: Dict[str, Any]

class BinaryContent(Content):
    """二进制内容"""
    type: ContentType = ContentType.BINARY
    data: bytes
    filename: Optional[str] = None

ContentUnion = Union[TextContent, ImageContent, JsonContent, BinaryContent]

class ToolExecutionRequest(BaseModel):
    """工具执行请求模型"""
    tool_name: str
    parameters: Dict[str, Any]
    timeout: Optional[float] = Field(None, description="执行超时时间(秒)")

class ToolExecutionResponse(BaseModel):
    """工具执行响应模型"""
    result: Optional[List[ContentUnion]] = None
    status: str = "success"
    error: Optional[str] = None

class ClientRegistrationRequest(BaseModel):
    """客户端注册请求"""
    agent_id: Optional[str] = None  # 可选的agent_id
    service_names: List[str]

class ServiceRegistrationStatus(BaseModel):
    """服务注册状态"""
    status: str  # success 或 failed
    message: Optional[str] = None
    error: Optional[str] = None

class ClientRegistrationResponse(BaseModel):
    """客户端注册响应"""
    agent_id: str  # 返回传入的或新生成的agent_id
    clients: Dict[str, Dict[str, Any]]  # client_id -> client_config
    services_status: Dict[str, ServiceRegistrationStatus]  # service_name -> status
    total_success: int
    total_failed: int
    status: str = "success"
    message: Optional[str] = None




from pydantic import BaseModel
from typing import Optional, List, Dict, Any

class ToolInfo(BaseModel):
    name: str
    description: str
    service_name: str
    client_id: Optional[str] = None
    inputSchema: Optional[Dict[str, Any]] = None

class ToolsResponse(BaseModel):
    tools: List[ToolInfo]
    total_tools: int

class ToolExecutionRequest(BaseModel):
    tool_name: str
    args: Dict[str, Any]
    agent_id: Optional[str] = None
    client_id: Optional[str] = None

class ToolExecutionResponse(BaseModel):
    success: bool
    result: Any
    error: Optional[str] = None 



"""
MCPStore 使用示例
展示如何使用新的基于上下文的 API
"""

import asyncio
from mcpstore.core.store import MCPStore
from mcpstore.core.orchestrator import MCPOrchestrator
from mcpstore.core.registry import ServiceRegistry
from mcpstore.plugins.json_mcp import MCPConfig

async def main():
    print("\n===== MCPStore 使用示例 (新版API) =====\n")

    # === 1. 初始化 ===
    print("1. 初始化 MCPStore")
    registry = ServiceRegistry()
    orchestrator = MCPOrchestrator({
        "timing": {
            "heartbeat_interval_seconds": 60,
            "heartbeat_timeout_seconds": 180,
            "http_timeout_seconds": 10,
            "command_timeout_seconds": 10
        }
    }, registry)
    mcp_config = MCPConfig()
    store = MCPStore(orchestrator, mcp_config)
    print("   ✓ 初始化完成")

    # === 2. 商店级别操作 ===
    print("\n2. 商店级别操作示例")
    
    # 2.1 使用链式调用
    print("\n2.1 链式调用方式")
    all_services = await store.for_store().list_services()
    print(f"   ✓ 获取到 {len(all_services)} 个服务")
    
    await store.for_store().add_service(['weather', 'maps'])
    print("   ✓ 添加服务成功")
    
    # 2.2 保存上下文重用
    print("\n2.2 保存上下文重用")
    store_ctx = store.for_store()
    services = await store_ctx.list_services()
    tools = await store_ctx.list_tools()
    health = store_ctx.check_services()
    print(f"   ✓ 商店共有 {len(services)} 个服务, {len(tools)} 个工具")
    print(f"   ✓ 服务健康状态: {health}")

    # === 3. Agent级别操作 ===
    print("\n3. Agent级别操作示例")
    
    # 3.1 链式调用
    print("\n3.1 链式调用方式")
    agent_id = "test_agent_123"
    agent_services = await store.for_agent(agent_id).list_services()
    print(f"   ✓ Agent订阅了 {len(agent_services)} 个服务")
    
    await store.for_agent(agent_id).add_service(['news'])
    print("   ✓ Agent订阅新服务成功")
    
    # 3.2 保存上下文重用
    print("\n3.2 保存上下文重用")
    agent_ctx = store.for_agent(agent_id)
    my_services = await agent_ctx.list_services()
    my_tools = await agent_ctx.list_tools()
    my_health = agent_ctx.check_services()
    print(f"   ✓ Agent可用服务: {len(my_services)}")
    print(f"   ✓ Agent可用工具: {len(my_tools)}")
    print(f"   ✓ Agent服务健康状态: {my_health}")

    # === 4. 工具使用 ===
    print("\n4. 工具使用示例")
    result = await store.use_tool('get_weather', {'city': '北京'})
    print(f"   ✓ 工具调用结果: {result}")

    print("\n===== 示例完成 =====")

if __name__ == "__main__":
    asyncio.run(main()) 



from fastapi import APIRouter, Depends, HTTPException, Path
from typing import Dict, Any, Optional, List
from mcpstore.core.store import McpStore
from mcpstore.scripts.models import (
    RegisterRequestUnion, JsonRegistrationResponse, JsonUpdateRequest, JsonConfigResponse,
    ServiceRegistrationResult
)
from mcpstore.scripts.deps import get_store
import logging
import time
import json
import os
from mcpstore.scripts.client_registration import load_client_configs, load_agent_clients

router = APIRouter()
logger = logging.getLogger(__name__)

@router.post("/register", response_model=Dict[str, str])
async def register_service(
    payload: RegisterRequestUnion,
    agent_id: Optional[str] = None,
    store: McpStore = Depends(get_store)
):
    return await store.register_service(payload, agent_id)

@router.post("/register/json", response_model=JsonRegistrationResponse)
async def register_json_service(
    client_id: Optional[str] = None,
    service_names: Optional[List[str]] = None,
    store: McpStore = Depends(get_store)
):
    return await store.register_json_service(client_id, service_names)

@router.put("/register/json", response_model=JsonRegistrationResponse)
async def update_json_service(
    payload: JsonUpdateRequest,
    store: McpStore = Depends(get_store)
):
    return await store.update_json_service(payload)

@router.get("/register/json", response_model=JsonConfigResponse)
async def get_json_config(
    client_id: Optional[str] = None,
    store: McpStore = Depends(get_store)
):
    return store.get_json_config(client_id)

@router.get("/health", response_model=Dict[str, Any])
async def get_health_status(store: McpStore = Depends(get_store)):
    return await store.get_health_status()

@router.get("/service_info", response_model=Dict[str, Any])
async def get_service_info(name: str, store: McpStore = Depends(get_store)):
    return store.get_service_info(name)

@router.get("/services")
async def list_services(store: McpStore = Depends(get_store)):
    return store.list_services()

@router.get("/tools")
async def list_tools(store: McpStore = Depends(get_store)):
    return store.list_tools()




import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

"""
MCP服务编排器

该模块提供了MCPOrchestrator类，用于管理MCP服务的连接、工具调用和查询处理。
它是FastAPI应用程序的核心组件，负责协调客户端和服务之间的交互。
"""

import asyncio
import logging
from typing import Dict, List, Any, Optional, Tuple, Set, Union, AsyncGenerator
from datetime import datetime, timedelta
from urllib.parse import urljoin

from mcpstore.core.registry import ServiceRegistry
from mcpstore.core.client_manager import ClientManager
from fastmcp import Client
from fastmcp.client.transports import (
    MCPConfigTransport,
    StreamableHttpTransport,
    SSETransport,
    PythonStdioTransport,
    NodeStdioTransport,
    UvxStdioTransport,
    NpxStdioTransport
)
from mcpstore.plugins.json_mcp import MCPConfig
from mcpstore.core.models.service import TransportType, ServiceRegistrationResult
from mcpstore.core.session_manager import SessionManager

logger = logging.getLogger(__name__)

class MCPOrchestrator:
    """
    MCP服务编排器

    负责管理服务连接、工具调用和查询处理。
    """

    def __init__(self, config: Dict[str, Any], registry: ServiceRegistry):
        """
        初始化MCP编排器

        Args:
            config: 配置字典
            registry: 服务注册表实例
        """
        self.config = config
        self.registry = registry
        self.clients: Dict[str, Client] = {}  # key为mcpServers的服务名
        self.main_client: Optional[Client] = None
        self.main_client_ctx = None  # async context manager for main_client
        self.main_config = {"mcpServers": {}}  # 中央配置
        self.agent_clients: Dict[str, Client] = {}  # agent_id -> client映射
        self.pending_reconnection: Set[str] = set()
        self.react_agent = None

        # 从配置中获取心跳和重连设置
        timing_config = config.get("timing", {})
        self.heartbeat_interval = timedelta(seconds=int(timing_config.get("heartbeat_interval_seconds", 60)))
        self.heartbeat_timeout = timedelta(seconds=int(timing_config.get("heartbeat_timeout_seconds", 180)))
        self.reconnection_interval = timedelta(seconds=int(timing_config.get("reconnection_interval_seconds", 60)))
        self.http_timeout = int(timing_config.get("http_timeout_seconds", 10))

        # 监控任务
        self.heartbeat_task = None
        self.reconnection_task = None
        self.mcp_config = MCPConfig()

        # 客户端管理器
        self.client_manager = ClientManager()

        # 会话管理器
        self.session_manager = SessionManager()

    async def setup(self):
        """初始化编排器资源（不再做服务注册）"""
        logger.info("Setting up MCP Orchestrator...")
        # 只做必要的资源初始化
        pass

    async def start_monitoring(self):
        """启动后台健康检查和重连监视器"""
        logger.info("Starting monitoring tasks...")

        # 启动心跳监视器
        if self.heartbeat_task is None or self.heartbeat_task.done():
            logger.info(f"Starting heartbeat monitor. Interval: {self.heartbeat_interval.total_seconds()}s")
            self.heartbeat_task = asyncio.create_task(self._heartbeat_loop())

        # 启动重连监视器
        if self.reconnection_task is None or self.reconnection_task.done():
            logger.info(f"Starting reconnection monitor. Interval: {self.reconnection_interval.total_seconds()}s")
            self.reconnection_task = asyncio.create_task(self._reconnection_loop())

    async def _heartbeat_loop(self):
        """后台循环，用于定期健康检查"""
        while True:
            await asyncio.sleep(self.heartbeat_interval.total_seconds())
            await self._check_services_health()

    async def _check_services_health(self):
        """检查所有服务的健康状态"""
        logger.debug("Running periodic health check for all services...")
        for client_id, services in self.registry.sessions.items():
            for name in services:
                try:
                    is_healthy = await self.is_service_healthy(name, client_id)
                    if is_healthy:
                        logger.debug(f"Health check SUCCESS for: {name} (client_id={client_id})")
                        self.registry.update_service_health(client_id, name)
                    else:
                        logger.warning(f"Health check FAILED for {name} (client_id={client_id})")
                        self.pending_reconnection.add(name)
                except Exception as e:
                    logger.warning(f"Health check error for {name} (client_id={client_id}): {e}")
                    self.pending_reconnection.add(name)

    async def _reconnection_loop(self):
        """定期尝试重新连接服务的后台循环"""
        while True:
            await asyncio.sleep(self.reconnection_interval.total_seconds())
            await self._attempt_reconnections()

    async def _attempt_reconnections(self):
        """尝试重新连接所有待重连的服务"""
        if not self.pending_reconnection:
            return  # 如果没有待重连的服务，跳过

        # 创建副本以避免迭代过程中修改集合的问题
        names_to_retry = list(self.pending_reconnection)
        logger.info(f"Attempting to reconnect {len(names_to_retry)} service(s): {names_to_retry}")

        for name in names_to_retry:
            try:
                # 尝试重新连接
                success, message = await self.connect_service(name)
                if success:
                    logger.info(f"Reconnection successful for: {name}")
                    self.pending_reconnection.discard(name)
                else:
                    logger.warning(f"Reconnection attempt failed for {name}: {message}")
                    # 保持name在pending_reconnection中，等待下一个周期
            except Exception as e:
                logger.warning(f"Reconnection attempt failed for {name}: {e}")

    async def connect_service(self, name: str, url: str = None) -> Tuple[bool, str]:
        """
        连接到指定的服务

        Args:
            name: 服务名称
            url: 服务URL（可选，如果不提供则从配置中获取）

        Returns:
            Tuple[bool, str]: (是否成功, 消息)
        """
        try:
            # 获取服务配置
            service_config = self.mcp_config.get_service_config(name)
            if not service_config:
                return False, f"Service configuration not found for {name}"

            # 如果提供了URL，更新配置
            if url:
                service_config["url"] = url

            # 创建新的客户端
            client = Client({"mcpServers": {name: service_config}})

            # 尝试连接
            try:
                await client.list_tools()
                self.clients[name] = client
                logger.info(f"Service {name} connected successfully")
                return True, "Connected successfully"
            except Exception as e:
                logger.error(f"Failed to connect to service {name}: {e}")
                return False, str(e)

        except Exception as e:
            logger.error(f"Failed to connect service {name}: {e}")
            return False, str(e)

    async def disconnect_service(self, url_or_name: str) -> bool:
        """从配置中移除服务并更新main_client"""
        logger.info(f"Removing service: {url_or_name}")

        # 查找要移除的服务名
        name_to_remove = None
        for name, server in self.main_config.get("mcpServers", {}).items():
            if name == url_or_name or server.get("url") == url_or_name:
                name_to_remove = name
                break

        if name_to_remove:
            # 从main_config中移除
            if name_to_remove in self.main_config["mcpServers"]:
                del self.main_config["mcpServers"][name_to_remove]

            # 从配置文件中移除
            ok = self.mcp_config.remove_service(name_to_remove)
            if not ok:
                logger.warning(f"Failed to remove service {name_to_remove} from configuration file")

            # 从registry中移除
            self.registry.remove_service(name_to_remove)

            # 重新创建main_client
            if self.main_config.get("mcpServers"):
                self.main_client = Client(self.main_config)

                # 更新所有agent_clients
                for agent_id in list(self.agent_clients.keys()):
                    self.agent_clients[agent_id] = Client(self.main_config)
                    logger.info(f"Updated client for agent {agent_id} after removing service")

            else:
                # 如果没有服务了，清除main_client
                self.main_client = None
                # 清除所有agent_clients
                self.agent_clients.clear()

            return True
        else:
            logger.warning(f"Service {url_or_name} not found in configuration.")
            return False

    async def refresh_services(self):
        """手动刷新所有服务连接（重新加载mcp.json）"""
        await self.load_from_config()

    async def is_service_healthy(self, name: str, client_id: Optional[str] = None) -> bool:
        """
        检查服务是否健康
        
        Args:
            name: 服务名
            client_id: 可选的客户端ID，用于多客户端环境
            
        Returns:
            bool: 服务是否健康
        """
        try:
            # 获取服务配置
            service_config = self.mcp_config.get_service_config(name)
            if not service_config:
                logger.warning(f"Service configuration not found for {name}")
                return False
            
            # 创建新的客户端实例
            client = Client({"mcpServers": {name: service_config}})
            
            try:
                # 使用超时控制的异步上下文管理器
                async with asyncio.timeout(self.http_timeout):
                    async with client:
                        await client.ping()
                        return True
            except asyncio.TimeoutError:
                logger.warning(f"Health check timeout for {name} (client_id={client_id})")
                return False
            except Exception as e:
                logger.warning(f"Health check failed for {name} (client_id={client_id}): {e}")
                return False
            finally:
                # 确保客户端被正确关闭
                try:
                    await client.close()
                except Exception:
                    pass  # 忽略关闭时的错误
                    
        except Exception as e:
            logger.warning(f"Health check failed for {name} (client_id={client_id}): {e}")
            return False

    # async def process_unified_query(
    #     self,
    #     query: str,
    #     agent_id: Optional[str] = None,
    #     mode: str = "react",
    #     include_trace: bool = False
    # ) -> Union[str, Dict[str, Any]]:
    #     """处理统一查询"""
    #     # 获取或创建会话
    #     session = self.session_manager.get_or_create_session(agent_id)
    #
    #     if not session.tools:
    #         # 如果会话没有工具，加载所有可用工具
    #         for service_name, client in self.clients.items():
    #             try:
    #                 tools = await client.list_tools()
    #                 for tool in tools:
    #                     session.add_tool(tool.name, {
    #                         "name": tool.name,
    #                         "description": tool.description,
    #                         "inputSchema": tool.inputSchema if hasattr(tool, "inputSchema") else None
    #                     }, service_name)
    #                     session.add_service(service_name, client)
    #             except Exception as e:
    #                 logger.error(f"Failed to load tools from service {service_name}: {e}")
    #
    #     # 处理查询...
    #     return {"result": "query processed", "session_id": session.agent_id}

    async def execute_tool(
        self,
        service_name: str,
        tool_name: str,
        parameters: Dict[str, Any],
        agent_id: Optional[str] = None
    ) -> Any:
        """执行工具"""
        try:
            if agent_id:
                # agent模式：在agent的所有client中查找服务
                client_ids = self.client_manager.get_agent_clients(agent_id)
                if not client_ids:
                    raise Exception(f"No clients found for agent {agent_id}")
                    
                # 在所有client中查找服务
                for client_id in client_ids:
                    if self.registry.has_service(client_id, service_name):
                        # 获取服务配置
                        service_config = self.mcp_config.get_service_config(service_name)
                        if not service_config:
                            logger.warning(f"Service configuration not found for {service_name}")
                            continue
                            
                        logger.debug(f"Creating new client for service {service_name} with config: {service_config}")
                        # 创建新的客户端实例
                        client = Client({"mcpServers": {service_name: service_config}})
                        try:
                            async with client:
                                logger.debug(f"Client connected: {client.is_connected()}")
                                
                                # 获取工具列表并打印
                                tools = await client.list_tools()
                                logger.debug(f"Available tools for service {service_name}: {[t.name for t in tools]}")
                                
                                # 检查工具名称格式
                                base_tool_name = tool_name
                                if tool_name.startswith(f"{service_name}_"):
                                    base_tool_name = tool_name[len(service_name)+1:]
                                logger.debug(f"Using base tool name: {base_tool_name}")
                                
                                # 检查工具是否存在
                                if not any(t.name == base_tool_name for t in tools):
                                    logger.warning(f"Tool {base_tool_name} not found in available tools")
                                    continue
                                
                                # 执行工具
                                logger.debug(f"Calling tool {base_tool_name} with parameters: {parameters}")
                                result = await client.call_tool(base_tool_name, parameters)
                                logger.info(f"Tool {base_tool_name} executed successfully with client {client_id}")
                                return result
                        except Exception as e:
                            logger.error(f"Failed to execute tool with client {client_id}: {e}")
                            continue
                                
                raise Exception(f"Service {service_name} not found in any client for agent {agent_id}")
            else:
                # store模式：在main_client的所有client中查找服务
                client_ids = self.client_manager.get_agent_clients(self.client_manager.main_client_id)
                if not client_ids:
                    raise Exception("No clients found in main_client")
                    
                # 在所有client中查找服务
                for client_id in client_ids:
                    if self.registry.has_service(client_id, service_name):
                        # 获取服务配置
                        service_config = self.mcp_config.get_service_config(service_name)
                        if not service_config:
                            logger.warning(f"Service configuration not found for {service_name}")
                            continue
                            
                        logger.debug(f"Creating new client for service {service_name} with config: {service_config}")
                        # 创建新的客户端实例
                        client = Client({"mcpServers": {service_name: service_config}})
                        try:
                            async with client:
                                logger.debug(f"Client connected: {client.is_connected()}")
                                
                                # 获取工具列表并打印
                                tools = await client.list_tools()
                                logger.debug(f"Available tools for service {service_name}: {[t.name for t in tools]}")
                                
                                # 检查工具名称格式
                                base_tool_name = tool_name
                                if tool_name.startswith(f"{service_name}_"):
                                    base_tool_name = tool_name[len(service_name)+1:]
                                logger.debug(f"Using base tool name: {base_tool_name}")
                                
                                # 检查工具是否存在
                                if not any(t.name == base_tool_name for t in tools):
                                    logger.warning(f"Tool {base_tool_name} not found in available tools")
                                    continue
                                
                                # 执行工具
                                logger.debug(f"Calling tool {base_tool_name} with parameters: {parameters}")
                                result = await client.call_tool(base_tool_name, parameters)
                                logger.info(f"Tool {base_tool_name} executed successfully with client {client_id}")
                                return result
                        except Exception as e:
                            logger.error(f"Failed to execute tool with client {client_id}: {e}")
                            continue
                                
                raise Exception(f"Tool not found: {tool_name}")
        except Exception as e:
            logger.error(f"Tool execution failed: {e}")
            raise Exception(f"Tool execution failed: {str(e)}")

    async def cleanup(self):
        """清理资源"""
        logger.info("Cleaning up MCP Orchestrator resources...")

        # 清理会话
        self.session_manager.cleanup_expired_sessions()

        # 停止监控任务
        if self.heartbeat_task and not self.heartbeat_task.done():
            self.heartbeat_task.cancel()
            try:
                await self.heartbeat_task
            except asyncio.CancelledError:
                pass

        if self.reconnection_task and not self.reconnection_task.done():
            self.reconnection_task.cancel()
            try:
                await self.reconnection_task
            except asyncio.CancelledError:
                pass

        # 关闭所有客户端连接
        for name, client in self.clients.items():
            try:
                await client.close()
            except Exception as e:
                logger.error(f"Error closing client {name}: {e}")

        self.clients.clear()
        self.pending_reconnection.clear()

    async def register_agent_client(self, agent_id: str, config: Optional[Dict[str, Any]] = None) -> Client:
        """
        为agent注册一个新的client实例

        Args:
            agent_id: 代理ID
            config: 可选的配置，如果为None则使用main_config

        Returns:
            新创建的Client实例
        """
        # 使用main_config或提供的config创建新的client
        agent_config = config or self.main_config
        agent_client = Client(agent_config)

        # 存储agent_client
        self.agent_clients[agent_id] = agent_client
        logger.info(f"Registered agent client for {agent_id}")

        return agent_client

    def get_agent_client(self, agent_id: str) -> Optional[Client]:
        """
        获取agent的client实例

        Args:
            agent_id: 代理ID

        Returns:
            Client实例或None
        """
        return self.agent_clients.get(agent_id)

    async def filter_healthy_services(self, services: List[str], client_id: Optional[str] = None) -> List[str]:
        """
        过滤出健康的服务列表

        Args:
            services: 服务名列表
            client_id: 可选的客户端ID，用于多客户端环境

        Returns:
            List[str]: 健康的服务名列表
        """
        healthy_services = []
        for name in services:
            try:
                service_config = self.mcp_config.get_service_config(name)
                if not service_config:
                    logger.warning(f"Service configuration not found for {name}")
                    continue

                # 创建新的客户端实例
                client = Client({"mcpServers": {name: service_config}})
                
                try:
                    # 使用超时控制的异步上下文管理器
                    async with asyncio.timeout(self.http_timeout):
                        async with client:
                            await client.ping()
                            healthy_services.append(name)
                except asyncio.TimeoutError:
                    logger.warning(f"Health check timeout for {name} (client_id={client_id})")
                    continue
                except Exception as e:
                    logger.warning(f"Health check failed for {name} (client_id={client_id}): {e}")
                    continue
                finally:
                    # 确保客户端被正确关闭
                    try:
                        await client.close()
                    except Exception:
                        pass  # 忽略关闭时的错误
                        
            except Exception as e:
                logger.warning(f"Health check failed for {name} (client_id={client_id}): {e}")
                continue

        return healthy_services

    async def start_main_client(self, config: Dict[str, Any]):
        """启动 main_client 的 async with 生命周期，注册服务和工具（仅健康服务）"""
        # 获取健康的服务列表
        healthy_services = await self.filter_healthy_services(list(config.get("mcpServers", {}).keys()))
        
        # 创建一个新的配置，只包含健康的服务
        healthy_config = {
            "mcpServers": {
                name: config["mcpServers"][name]
                for name in healthy_services
            }
        }
        
        # 使用健康的配置注册服务
        await self.register_json_services(healthy_config, client_id="main_client")
        # main_client专属管理逻辑可在这里补充（如缓存、生命周期等）

    async def register_json_services(self, config: Dict[str, Any], client_id: str = None, agent_id: str = None):
        """注册JSON配置中的服务（可用于main_client或普通client）"""
        # agent_id 兼容
        agent_key = agent_id or client_id or self.client_manager.main_client_id
        try:
            # 获取健康的服务列表
            healthy_services = await self.filter_healthy_services(list(config.get("mcpServers", {}).keys()), client_id)
            
            if not healthy_services:
                logger.warning("No healthy services found")
                return {
                    "client_id": client_id or "main_client",
                    "services": {},
                    "total_success": 0,
                    "total_failed": 0
                }

            # 使用healthy_services构建新的配置
            healthy_config = {
                "mcpServers": {
                    name: config["mcpServers"][name]
                    for name in healthy_services
                }
            }
            
            # 使用健康的配置创建客户端
            client = Client(healthy_config)

            try:
                async with client:
                    # 获取工具列表
                    tool_list = await client.list_tools()
                    if not tool_list:
                        logger.warning("No tools found")
                        return {
                            "client_id": client_id or "main_client",
                            "services": {},
                            "total_success": 0,
                            "total_failed": 0
                        }

                    # 处理工具列表
                    all_tools = []
                    
                    # 判断是否是单服务情况
                    is_single_service = len(healthy_services) == 1
                    
                    for tool in tool_list:
                        tool_name = tool.name
                        
                        # 确定工具所属的服务
                        if is_single_service:
                            # 单服务情况：所有工具都属于这个服务
                            service_name = healthy_services[0]
                            # 如果工具名称还没有服务前缀，添加前缀
                            if not tool_name.startswith(f"{service_name}_"):
                                tool_name = f"{service_name}_{tool_name}"
                        else:
                            # 多服务情况：根据工具名称前缀判断
                            service_name = None
                            for name in healthy_services:
                                if tool_name.startswith(f"{name}_"):
                                    service_name = name
                                    break
                                    
                            if not service_name:
                                logger.warning(f"Tool {tool_name} does not belong to any service, skipping")
                                continue

                        # 处理参数信息
                        parameters = {}
                        if hasattr(tool, 'inputSchema') and tool.inputSchema:
                            parameters = tool.inputSchema
                        elif hasattr(tool, 'parameters') and tool.parameters:
                            parameters = tool.parameters

                        tool_def = {
                            "type": "function",
                            "function": {
                                "name": tool_name,  # 使用可能被修改过的tool_name
                                "description": tool.description,
                                "parameters": parameters
                            }
                        }
                        all_tools.append((tool_name, tool_def))  # 使用可能被修改过的tool_name

                    # 为每个服务注册其工具
                    for service_name in healthy_services:
                        if is_single_service:
                            service_tools = all_tools
                        else:
                            service_tools = [(name, tool_def) for name, tool_def in all_tools if name.startswith(f"{service_name}_")]
                        logger.info(f"Filtered {len(service_tools)} tools for service {service_name}")
                        self.registry.add_service(agent_key, service_name, client, service_tools)
                        self.clients[service_name] = client

                    return {
                        "client_id": client_id or "main_client",
                        "services": {
                            name: {"status": "success", "message": "Service registered successfully"}
                            for name in healthy_services
                        },
                        "total_success": len(healthy_services),
                        "total_failed": 0
                    }
            except Exception as e:
                logger.error(f"Error retrieving tools: {e}", exc_info=True)
                return {
                    "client_id": client_id or "main_client",
                    "services": {},
                    "total_success": 0,
                    "total_failed": 1,
                    "error": str(e)
                }
        except Exception as e:
            logger.error(f"Error registering services: {e}", exc_info=True)
            return {
                "client_id": client_id or "main_client",
                "services": {},
                "total_success": 0,
                "total_failed": 1,
                "error": str(e)
            }

    def create_client_config_from_names(self, service_names: list) -> Dict[str, Any]:
        """
        根据服务名列表，从 mcp.json 生成新的 client config
        """
        all_services = self.mcp_config.load_config().get("mcpServers", {})
        selected = {name: all_services[name] for name in service_names if name in all_services}
        return {"mcpServers": selected}

    def remove_service(self, service_name: str, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        self.registry.remove_service(agent_key, service_name)
        # ...其余逻辑...

    def get_session(self, service_name: str, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        return self.registry.get_session(agent_key, service_name)

    def get_tools_for_service(self, service_name: str, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        return self.registry.get_tools_for_service(agent_key, service_name)

    def get_all_service_names(self, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        return self.registry.get_all_service_names(agent_key)

    def get_all_tool_info(self, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        return self.registry.get_all_tool_info(agent_key)

    def get_service_details(self, service_name: str, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        return self.registry.get_service_details(agent_key, service_name)

    def update_service_health(self, service_name: str, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        self.registry.update_service_health(agent_key, service_name)

    def get_last_heartbeat(self, service_name: str, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        return self.registry.get_last_heartbeat(agent_key, service_name)

    def has_service(self, service_name: str, agent_id: str = None):
        agent_key = agent_id or self.client_manager.main_client_id
        return self.registry.has_service(agent_key, service_name)



import logging
import os
import sys
import time
import uuid
import json # for pretty printing
from fastapi import Request
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.exceptions import RequestValidationError
from mcpstore.scripts.exception_handlers import validation_exception_handler

from mcpstore.scripts.deps import app_state
from plugins.json_mcp import MCPConfig
from core.registry import ServiceRegistry
from core.orchestrator import MCPOrchestrator
from core.store import McpStore
from core.client_manager import ClientManager
from core.session_manager import SessionManager
from mcpstore.scripts import service_api, client_api, tool_api
from typing import Callable
from starlette.middleware.base import BaseHTTPMiddleware

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                    handlers=[logging.StreamHandler(sys.stdout), logging.FileHandler("mcp_service.log")])
logger = logging.getLogger(__name__)
logger.info("【第8步】Uvicorn 正在导入 app.py 文件。")

async def lifespan(app: FastAPI):
    logger.info("【第10步】FastAPI 的 lifespan 已启动，开始初始化核心组件。")

    config_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "data", "mcp.json")
    mcp_config_handler = MCPConfig(config_path)
    logger.info(f"    - MCPConfig 实例已创建，配置文件路径: {config_path}")

    config = mcp_config_handler.load_config()
    logger.info("    - 配置文件 mcp.json 已加载。")
    logger.info(f"    - 加载的配置内容: \n{json.dumps(config, indent=2, ensure_ascii=False)}")

    registry = ServiceRegistry()
    logger.info("    - ServiceRegistry 实例已创建。")

    orchestrator = MCPOrchestrator(config=config, registry=registry)
    logger.info("    - MCPOrchestrator 实例已创建。")

    store = McpStore(orchestrator=orchestrator, config=mcp_config_handler)
    logger.info("    - McpStore 实例已创建，聚合了所有核心组件。")
    logger.info("【第11步】所有核心组件的唯一实例已创建完毕。")

    logger.info("    - 准备调用 orchestrator.setup()")
    await orchestrator.setup()
    logger.info("    - orchestrator.setup() 已完成。")

    # logger.info("    - 准备调用 orchestrator.start_monitoring()")
    # await orchestrator.start_monitoring()
    # logger.info("    - orchestrator.start_monitoring() 已完成，后台健康检查等任务已启动。")

    # logger.info("    - 准备调用 orchestrator.register_json_services()，注册 mcp.json 中的服务。")
    # registration_results = await orchestrator.register_json_services(config, client_id="main_client")
    # logger.info("    - orchestrator.register_json_services() 已完成。")
    # logger.info(f"    - 服务注册结果: \n{json.dumps(registration_results, indent=2, ensure_ascii=False)}")

    app_state["store"] = store
    logger.info("    - 唯一的 McpStore 实例已存入 app_state。")
    logger.info("【第12步】应用启动流程 (lifespan) 即将完成，准备移交控制权。")

    try:
        yield
        logger.info("Lifespan 正常结束，应用即将关闭。")
    finally:
        logger.info("Application shutdown: Cleaning up resources...")
        orch = app_state.get("orchestrator")
        if orch:
            await orch.stop_main_client()
            await orch.cleanup()
        app_state.clear()
        logger.info("Application shutdown complete.")

# 创建应用实例
app = FastAPI(lifespan=lifespan)
logger.info("【第9步】FastAPI 应用实例 'app' 已创建。")

# 配置CORS
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# 注册路由
app.include_router(service_api.router, tags=["services"])
app.include_router(client_api.router, tags=["clients"])
app.include_router(tool_api.router, tags=["tools"])

# 注册异常处理
app.add_exception_handler(RequestValidationError, validation_exception_handler)

# 添加请求日志中间件
@app.middleware("http")
async def log_requests(request: Request, call_next):
    """
    Middleware to log incoming requests, processing time, and status.
    """
    request_id = str(uuid.uuid4())
    logger.info(f"Request received - ID: {request_id}, Method: {request.method}, Path: {request.url.path}")
    start_time = time.time()
    
    try:
        response = await call_next(request)
        process_time = (time.time() - start_time) * 1000
        logger.info(
            f"Request finished - ID: {request_id}, "
            f"Status: {response.status_code}, Duration: {process_time:.2f}ms"
        )
        return response
    except Exception as e:
        process_time = (time.time() - start_time) * 1000
        logger.error(
            f"Request failed - ID: {request_id}, "
            f"Error: {e}, Duration: {process_time:.2f}ms",
            exc_info=True
        )
        raise 
